# awk script to create notelet pages from formatted text
# v9.1 2024/05/04 k.j.beeden
# nb. change version text at BEGIN
# awk -f bullets.awk bullets.fmt <pagesize> <plain|spine> <[t]> <[embed]> > bullets.ps
#
# options:
# 	<pagesize> is mandatory, and can be [a4|a5|a6|a6p|a7|b7|f5|f6|f6a|p4|p6|mi|m7|q7|A4|A5|A6|A6P|A7|B7|F5|F6|F6A|P4|P6|MI|M7|Q7]
#		lower case is booklet format
#		upper case is notelet
#		Ax		Ax paper
#		A6		A6 paper 6-hole (1.75" gap)
#		A6P		A6 paper Personal 6-hole (2" gap)
#		B7		Daiso Pocket B7 6-hole
#		F5	 	Filofax A5 Clipbook 6-hole
#		F6	 	Filofax Personal 6-hole (non-Ax page size)
#		F6A	 	Filofax PersonalA 6-hole (A6 page width)
#		P4		Filofax PocketA 4-hole
#		P6		Filofax PocketA 6-hole
#		MI		Filofax Mini (narrower A7) 5-hole
#		M7		Filofax MiniA (A7) 5-hole
#		Q7		QL binder (wider A7) 3-hole
#	<plain|spine> is mandatory
#		plain	no spine margin
#		spine	DofE section spine margin
#	<[t]>
#	 	[] 		= no tumble
#	 	t		= tumbled pages
#	<[embed]>
#		embed	embed JPEG images for direct PostScript printing

# Supported format strings
##########################
# document information & options
# <fs> source file name
# <fv> source file version
# <se> select section even page padding
# <bh> mark binding holes (except for A4 paper)
# <n[0|1]> <separator character> page numbering [simple|section-page]

# page type
# <p0> unnumbered page
# <p1> numbered page
# <pt> blank page for tumble double-sided

# text formatting
# <co> text colour selection

# printed elements
# text line format strings
# centred title text
# <t0> title 0
# <t1> title 1 <date> option
# <t2> title 2
# <fh> file history
#
# <ph> printing history
#
# left aligned text
# <c0> chapter heading (bold)
# <s0> section heading (bold)
# <h0> heading 0 (bold)
# <h1> heading 1 (bold)
# <h2> heading 2 (bold)
# <b0> bullet 0
# <b1> bullet 1
# <b2> bullet 2
# <i0> italic bullet 0
# <i1> italic bullet 1
# <i2> italic bullet 2
# <k0> bullet 0 with tick box
# <k1> bullet 1 with tick box
# <c1> contact phone number
# <fn> footnote

# graphic insertion format strings
# <l0> DofE logo
# <ge> EPS graphic insertion
# <gE3> EPS graphic insertion - folded 3 page
# <gj> JPEG graphic insertion - single page
# <gJ3> JPEG graphic insertion - folded 3 page

# table insertion format strings
# <or> [portrait|landscape]		table orientation & initialise extents within margins
# <st> (string)					spine text
# <tb> 							start table, numbered page
# <po> [line thickness (mm)]	draw table outline
# <ad> area_x area_y pos_x pos_y	define table area & position
# <ao> [line thickness (mm)]	draw area outline
# <rd> rows						define number of rows in this area
# <co> width1% [widthn%]]		define column width positions
# <cl> col start stop [start stop]	draw column lines between rows
# <bt> thickness					set border thickness (mm)
# <bf> colour					set border format (colour)
# <bs>							set border line style (cap, join, dash)
# <fi> row col <colourname|postscript colour definition>
# <fi> min_col max_col min_row max_row
# <ft> fontname					set text font
# <ff> size xoffset yoffset		fixed font size & offsets (mm)
# <fr> size% xoff% yoff%		relative font size & offsets (% of row height)
# <fa>							activate font as defined
# <tx>	[|<|>](string)[|>] row col1 [col2]
#	<(string)	left justified
#	<(string)>	centred
#	(string)>	right justified
#	if there are two col parameters, will position between the two
#		use for centre justified
#	>(string)	area title, flushed, at top of are
# <bj>							fill area with bujo dot grid
# <te> size_x size_y filename	insert EPS figure in area
# <tj> size_x size_y filename	insert JPEG figure in area


BEGIN {
	# #####################################
	# Setup
	# script version
	script_file		= "bullets.awk"
	script_version	= "v9.1"
	script_path		= "../scripts/" script_file

	# user colour settings (using definitions in 'colours.ps')
	# logo colour (p6 'DofE_Brand_Guidelines.pdf')
	dofe_colour		= "gunmetal"	# set colour of logo
	# text colour (p9 'DofE_Brand_Guidelines.pdf')
	body_colour		= "text"		# set colour of body text

	# get file dates
	get_dates()

	# Settings
	print_lines	= "<c0><s0><h0><h1><h2><b0><b1><b2><i0><i1><i2><k0><k1><c1>"	# format types that print a line of text
	max_lines	= 23		# maximum number of printable lines per page

	# extract settings from command line
	# set defaults
	sheet		= 0			# physical paper sheet
	page		= 0			# physical page, start on page 0
	number		= 0			# numbered page, start on page 0
	paper		= "A4"		# page size [A4|A5|A6|A6P|A7|B7|F5|F6|F6A|P4|P6|Q7|M7|MI]
	book		= 0			# select document format [0|1] notelet|booklet
	spine		= 0			# enable spine colour blocks [0|1]
	tumble		= 0			# enable even page split tumble [0|1]
	tumble_y	= 8			# vertical offset to allow for tumble printing
	embed		= 0			# embed JPEG images in output file (for direct PostScript printing) [0|1]

	# decode options
	decode_options()

	# set page properties
	set_page_properties()

	# page, font & logo definitions (based on A4 page)
	set_layout_options()

	# working variables
	format		= ""
	last_format	= ""
	text		= ""

	offset_y	= 0
	lines		= 0
	rot_page	= 0
	chap_num	= 0
	sect_num	= 0
	sect_page	= 0
	num_style	= 0
	pn_pend		= 0
	sect_even	= 0			# section even page padding
	do_binding	= 0			# enable printing of binding holes

	# table variables
	area		= 0		# table area counter
	cols		= 0
	rows		= 0

	# print PS file header and prolog
	print_ps_header()

	# set field separator to tab
	FS	= "\t"

}	# END setup


# #############################################
# Parse formatted text lines
{
	# get command and text string
	format	= $1
	text	= substr($0, index($0, ">")+2)

#	###################
	# parse embedded document options
	parse_doc_options(format, text)

	# process new document page
	new_page(format)

	# chapter or section odd page start
	section_page(format)

#	###################
	# process printable content lines

	# process lines for all pages, as selected by command line
	# cover features
	cover_features(format)

	# history features
	history_features(format)

	# footnote
	footnote(format, text)

	# printed body lines
	printed_lines(format, text)

	# implement table features
	table_features(format, text)
}


# #####################################
#
END {
	# spine & binding
	print "spine & binding"
	# print spine unless blank tumble page
	if ((spine == 1) && (format != "<pt>"))
	{
		print "paper spine"
		print exped_tab_x, "expedTab"
	}

	# if enabled, print binding holes every 8 sheets
	if (((sheet % 8) == 1) && (do_binding == 1))
	{
		print "paper binding"
	}

	# check if page number printing is pending, and this is a printing page
	if (pn_pend == 1)
	{
		print "% last page number"
		print "select_p1_font"
		print p1_x, p1_y, "moveto"
		# select page numbering style
		if (num_style == 1)
			printf("(%d%s%d) centreshow\n", sect_num, num_sep, sect_page)
		else
			printf("(%d) centreshow\n", number)
		# clear pending page number
		pn_pend		= 0
	}

	# create blank page to ensure even number of pages per section
	if ((page % 2) == 1)
	{
		new_page(page_format)
		# reset line counter
		lines	= 0
	}

	print "showpage"
	print ""
	print "%%Trailer"
	print "%%Pages:", sheet
}


# #####################################
#
function get_dates()
{
	dates = "JanFebMarAprMayJunJulAugSepOctNovDec"
	# get today's date in known format
	"date +'%a %b %d %T %m %Y'" | getline
	month		= $5
	day			= $3
	thisyear	= $6
	# date string for PostScript creation date (yyyy/mm/dd)
	today		= sprintf("%04d/%02d/%02d", thisyear, month, day)
	# date string for printing record (hh:mm:ss DOW DOM Mon yyyy)
	time		= sprintf("%s %s %s %s %s", $4, $1, $3, $2, $6)

	# get text filedate (can't use FILENAME variable, as doesn't exist in BEGIN section)
	command		= "ls --fu " ARGV[1]
	command | getline
	# check for current year
	year		= substr($6, 1, 4)
	month		= substr($6, 6, 2)
	day			= substr($6, 9, 2)
	filedate	= sprintf("%04d/%02d/%02d", year, month, day)

	# get script file date
	command 	= "ls --fu " script_path
	command | getline
	year		= substr($6, 1, 4)
	month		= substr($6, 6, 2)
	day			= substr($6, 9, 2)
	scriptdate	= sprintf("%04d/%02d/%02d", year, month, day)
}

function decode_options()
{
	if (ARGC > 2)
	{
		# decode A size paper
		if (("a4a5a6a7A4A5A6A7" ~ ARGV[2]) && (length(ARGV[2])==2))
		{
			paper = sprintf("A%d", substr(ARGV[2],2))
		}
		# decode A6P paper
		if ((ARGV[2] == "a6p") || (ARGV[2] == "A6P"))
			paper = "A6P"
		# decode B7 paper
		if ((ARGV[2] == "b7") || (ARGV[2] == "B7"))
			paper = "B7"
		# decode QL binder paper
		if ((ARGV[2] == "q7") || (ARGV[2] == "Q7"))
			paper = "Q7"
		# decode Filofax A5 six hole
		if ((ARGV[2] == "f5") || (ARGV[2] == "F5"))
			paper = "F5"
		# decode Filofax Personal six hole
		if ((ARGV[2] == "f6") || (ARGV[2] == "F6"))
			paper = "F6"
		# decode Filofax PersonalA six hole
		if ((ARGV[2] == "f6a") || (ARGV[2] == "F6A"))
			paper = "F6A"
		# decode Filofax Pocket four hole
		if ((ARGV[2] == "p4") || (ARGV[2] == "P4"))
			paper = "P4"
		# decode Filofax Pocket six hole
		if ((ARGV[2] == "p6") || (ARGV[2] == "P6"))
			paper = "P6"
		# decode Filofax MiniA 5-hole
		if ((ARGV[2] == "m7") || (ARGV[2] == "M7"))
			paper = "M7"
		# decode Filofax Mini 5-hole
		if ((ARGV[2] == "mi") || (ARGV[2] == "MI"))
			paper = "MI"

		# check for booklet cover
		if ("a4a5a6a6pb7f5f6p4pm7mi6q7" ~ ARGV[2])
		{
			book = 1
		}
		# check for spine option
		if (ARGC > 3)
			if (ARGV[3] == "spine")
				# enable DofE spine for online document
				spine	= 1

		# check to see if tumble or embed options set
		if (ARGC > 4)
		{
			# tumble even pages
			if ((ARGV[4] == "t") || (ARGV[5] == "t"))
				tumble	= 1
			#  embed JPEG images in output file
			if ((ARGV[4] == "embed") || (ARGV[5] == "embed"))
				embed	= 1
		}
		
		# leave only script name and filename in arguments array
		ARGC	= 2
	}
}

function set_page_properties()
{
	# default values
	page_x		= 210				# printed width
	page_y		= 297				# printed height
	max_lines	= 23
	Q7_bind		= 0
	ring_radius	= 2					# binding
	
	switch (paper)
	{
		case "A4":
			paper_x		= 210				# actual width
			paper_y		= 297				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 80				# centre spacing
			ring_space	= 80				# ring spacing
			ring_group	= 0					# #rings in each group
			ring_radius	= 3					# binding
			break							# end case
		case "A5":
			paper_x		= 148.5				# actual width
			paper_y		= 210				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 69.85				# centre spacing 2.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			ring_radius	= 2.5				# binding
			break							# end case
		case "A6":							# A6 6-hole 1.75" gap
			paper_x		= 105				# actual width
			paper_y		= 148.5				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 38.10				# centre spacing 1.5"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "A6P":							# A6 Filofax Personal punching 6-hole (2" gap)
			paper_x		= 105				# actual width
			paper_y		= 148.5				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "A7":							# A7 stitching
			paper_x		= 74.25				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 3.54				# ring offset 
			ring_gap	= 0					# centre spacing 0 
			ring_space	= 3.89				# ring spacing 
			ring_group	= 8					# #rings in each group
			ring_radius	= 0.177				# sewn binding
			break							# end case
		case "B7":
			paper_x		= 88.29				# actual width (A7 * 4rt(2))
			paper_y		= 124.87			# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 19.05				# centre spacing 0.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "F5":							# Filofax Clipbook A5
			paper_x		= 148.5				# actual width
			paper_y		= 210				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 69.85				# centre spacing 2.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "F6":							# Filofax Personal
			page_x		= 210				# printed width
			page_y		= 378				# printed height
			paper_x		= 95				# actual width		(Filofax Personal width 3.75", 95mm)
			paper_y		= 171				# actual height		(Filofax Personal height 6.75", 171mm)
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			max_lines	= 27
			break							# end case
		case "3F6":							# Filofax Triple page Personal
			page_x		= 590				# printed width		(Filofax Personal width 3.75", 95mm)
			page_y		= 378				# printed height	(Filofax Personal height 6.75", 171mm)
			paper_x		= 267				# actual width (95 + 2 * (95-9))
			paper_y		= 171				# actual height 
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			max_lines	= 27
			break							# end case
		case "F6A":							# Filofax PersonalA
			page_x		= 210				# printed width
			page_y		= 342				# printed height
			paper_x		= 105				# actual width		(A6 width)
			paper_y		= 171				# actual height		(Filofax Personal height 171mm)
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			max_lines	= 27
			break							# end case
		case "P4":							# Filofax PocketA 4-hole
			paper_x		= 85.31				# actual width
			paper_y		= 120.65			# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 2					# #rings in each group
			break							# end case
		case "P6":							# Filofax PocketA 6-hole
			paper_x		= 85.31				# actual width
			paper_y		= 120.65			# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 19.05				# centre spacing 0.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "Q7":							# QL wide A7 3-hole
			paper_x		= 79.90				# actual width
			paper_y		= 105				# actual height
			Q7_bind		= 16				# binding margin for use with QL ring binder
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 0					# centre spacing 0
			ring_space	= 38.10				# ring spacing 1.5"
			ring_group	= 2					# #rings in each group
			break							# end case
		case "M7":							# Filofax MiniA A7 5-hole
			paper_x		= 74.25				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 0					# centre spacing 0
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "MI":							# Filofax Mini narrow A7 5-hole
			page_x		= 190				# printed width
			page_y		= 297				# printed height
			paper_x		= 67.50				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 0					# centre spacing 0
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		default:
			paper_x		= 210				# actual width
			paper_y		= 297				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 80				# centre spacing
			ring_space	= 80				# ring spacing
			ring_group	= 0					# #rings in each group
			ring_radius	= 3					# binding
			break							# end case
    }

	# calculate paper scale
	paper_scale	= paper_y/page_y	# printed to actual scale
}

function set_layout_options()
{
	margin_x	= 20.0				# left margin
	margin_top	= 20.3				# top margin
	margin_bot	= 10				# bottom margin
	l0_font		= 153				# point height of raw logo
	l0_kern		= 116				# point width of raw logo
	if (book == 1)
	{
		l0_height	= page_x/4			# logo height
		t0_space	= -14.0				# title 0 line spacing
		t0_gap		= 28.5				# title 0 y position
	}
	else
	{
		l0_height	= l0_font/2			# logo height
		t0_space	= 39.0				# title 0 line spacing
		t0_gap		= 53.2				# title 0 y position
	}
	l0_gap		= l0_height+3.4		# logo y position
	t1_space	= 13.0				# title 0 line spacing
	t1_gap		= 186.7				# title 1 y position
	t2_space	= 8.77				# title 0 line spacing
	t2_gap		= 236.7				# title 2 y droop

	t_horiz		= page_x/2			# title x position centre
	c0_space	= 10.75				# chapter line spacing
	s0_space	= 10.75				# section line spacing
	h0_space	= 10.75				# header 0 line spacing
	h1_space	= 10.75				# header 1 line spacing
	h2_space	= 10.75				# header 2 line spacing
	b0_space	= 10.75				# bullet 0 line spacing
	b1_space	= 10.75				# bullet 1 line spacing
	b2_space	= 10.75				# bullet 2 line spacing
	tab_x		= 15.0				# tab width
	p1_x		= page_x - margin_x	# page marker position
	p1_y		= margin_bot		# page marker position
	wide_page	= 0					# wide page position offset
	ph_space	= 6					# print history line spacing
	ph_posn		= margin_bot		# print history position
	image_y		= page_y - margin_top - max_lines * b0_space

	# spine detail
	tabOffset 	= 0					# margin offset for section tab
	tabWidth	= 13.4				# width of section tab
	tabSpace	= 2.0				# space between section tabs
	tabHeight	= 8.25				# height of section tab
	titleHeight	= 5.92				# font height of title text

	# print & fold mark sizes
	tick		= 20				# length of grid tick lines
	fold		= 7					# length of fold lines

	# title & tab positions
	if (book == 1)
	{
		# booklet
		titleStart	= 274.05			# vertical position of title text
		stripeStart	= 278.25			# vertical position of bottom of award level stripes
		tabStart	= 21.25				# vertical position of bottom of section tabs
	}
	else
	{
		# notelet
		titleStart	= 253.75			# vertical position of title text
#		stripeStart	= page_y-margin_top-h0_space			# vertical position of bottom of award level stripes
		stripeStart	= 257.95			# fixed vertical position of bottom of award level stripes
		tabStart	= 32				# vertical position of bottom of section tabs
	}

	# contact details
	contStroke	= 0.2				# line width of contact underscore
	contWidth	= 90				# width of contact underscore

	# 3 award stripes and 2 gaps is same height as one tab
	awardSpace	= tabSpace/3
	awardHeight = (tabHeight-2*awardSpace)/3

	# table margins
	binding_margin	= margin_x
	page_margin		= tabWidth
	top_margin		= margin_top
	footer_margin	= margin_bot
}

function print_ps_header()
{
	print "%!PS-Adobe-3.0"
	print "%%Title:		DofE Aide Memoire bullet points"
	print "%%Creator:		bullets.awk", script_version, "copyright 2012-" thisyear, "K J Beeden"
	print "%%CreationDate:	" today
	if (paper == "Q7")
		printf("%%%%BoundingBox:	0 0 %d %d\n", (page_x + Q7_bind) * 72/25.4, page_y * 72/25.4)
	else
		printf("%%%%BoundingBox:	0 0 %d %d\n", page_x * 72/25.4, page_y * 72/25.4)
	print "%%Pages: (atend)"
	print "%%LanguageLevel:	3"
	print "%%EndComments"
	print "%%BeginProlog"

	# dump parameters
	print "% imposition parameters"
	print "/paper\t\t\t(" paper ") def"
	print "/a4_x\t\t\t" 210, "def"
	print "/a4_y\t\t\t" 297, "def"
	print "/page_x\t\t\t" page_x, "def"
	print "/page_y\t\t\t" page_y, "def"
	print "/paper_x\t\t" paper_x, "def"
	print "/paper_y\t\t" paper_y, "def"
	print "/paper_scale\t" paper_scale, "def"
	print "/b0_space\t\t" b0_space, "def"
	print "/Q7_bind\t\t" Q7_bind, "def"
	print "/ring_margin\t" ring_margin, "def"
	print "/ring_gap\t\t" ring_gap, "def"
	print "/ring_space\t\t" ring_space, "def"
	print "/ring_group\t\t" ring_group, "def"
	print "/ring_radius\t" ring_radius, "def"
	print ""

	print "% formatting parameters"
	print "/margin_x\t\t" margin_x, "def"
	print "/margin_top\t\t" margin_top, "def"
	print "/margin_bot\t\t" margin_bot, "def"
	print "/tab_x\t\t\t" tab_x, "def"
	print "/tumble_y\t\t" tumble_y, "def"
	print "/image_y\t\t" image_y, "def"
	print "/tick\t\t\t" tick, "def"
	print "/fold\t\t\t" fold, "def"
	print ""

	print "% table.awk formatting parameters"
	print "/binding_margin\t\t" binding_margin, "def"
	print "/page_margin\t\t" page_margin, "def"		# right page margin is expedition tab
	print "/footer_margin\t\t" footer_margin, "def"
	print "/top_margin\t\t\t" top_margin, "def"
	print "% page extent"
	print "/extent_x\t\t\t" page_x - binding_margin - page_margin, "def"
	print "/extent_y\t\t\t" page_y - footer_margin - top_margin, "def"
	print ""

	# include official colour scheme
	system("cat ../scripts/postscript/colours.ps")
	# add user-defined body text colour
	print "% user-defined body text colour"
	print "/body_colour	{" body_colour "} def"
	print ""

	# include fixed functions
	system("cat ../scripts/postscript/functions.ps")
	system("cat ../scripts/postscript/tables_hdr.eps")

	# re-map character coding to add copyright character
	system("cat ../scripts/postscript/fonts.ps")

	# include DofE logo
	system("cat ../scripts/postscript/logo.ps")

	print "% end of inserts"
	print ""

}

function parse_doc_options(format, text)
{
	# file details
	if (format == "<fs>")
		source_file	= text
	if (format == "<fv>")
		file_version	= text

	# page numbering style
	if (format == "<n0>")
	{
		num_style	= 0
		num_sep		= substr(text, 1, 1)
	}
	if (format == "<n1>")
	{
		num_style	= 1
		num_sep		= substr(text, 1, 1)
	}

	# section even page padding
	if (format == "<se>")
		sect_even	= 1

	# decode page setup options
	# margins
	# all margins, absolute, in mm
	if (format == "<mg>")
	{
		print "% margins"
		binding_margin	= $2 / paper_scale
		page_margin		= $3 / paper_scale
		footer_margin	= $4 / paper_scale
		top_margin		= $5 / paper_scale
		# derived values, scaled
		extent_x		= page_x - binding_margin - page_margin
		extent_y		= page_y - footer_margin - top_margin
		print "% page margins"
		print "/binding_margin	" binding_margin, "def"
		print "/page_margin	" page_margin, "def"
		print "/top_margin		" top_margin, "def"
		print "/footer_margin	" footer_margin, "def"
		print "% page extent"
		print "/extent_x		" extent_x, "def"
		print "/extent_y		" extent_y, "def"
	}

	# page orientation (portrait or landscape)
	if (format == "<or>")
	{
		# report margins
		print "% binding_margin\t" binding_margin
		print "% page_margin\t\t" page_margin
		print "% top_margin\t\t" top_margin
		print "% footer_margin\t\t" footer_margin

		# check to see if orientation already set within this page
		if (orientation != "")
		{
			# check to see if this a re-orientation from landscape within a page
			if ((orientation == "landscape") && ($2 == "portrait"))
			{
				print "% undo landscape orientation"
				if ((page % 2) == 1)
				{
					# odd page
					if (orientation == "landscape")
						print 0, extent_y, "translate 0 0 moveto -90 rotate"
				}
				else
				{
					# even page
					if (orientation == "landscape")
						print extent_x, 0, "translate 0 0 moveto 90 rotate"
				}
			}
			orientation		= $2
			print "% orientation", orientation
			# derive table extent
			if ($2 == "portrait")
			{
				extent_x		= page_x - binding_margin - page_margin
				extent_y		= page_y - footer_margin - top_margin
			}
			else
			{
				extent_y		= page_x - binding_margin - page_margin
				extent_x		= page_y - footer_margin - top_margin
			}
		}
		else
		{
			orientation		= $2
			print "% orientation", orientation
			# derive table extent
			if ($2 == "portrait")
			{
				extent_x		= page_x - binding_margin - page_margin
				extent_y		= page_y - footer_margin - top_margin
			}
			else
			{
				extent_y		= page_x - binding_margin - page_margin
				extent_x		= page_y - footer_margin - top_margin
			}
			print "%	extents", extent_x, extent_y
			if (orientation == "landscape")
			{
				print "% change to landscape"
				if ((page % 2) == 1)
				{
					# odd page
					print "0 0 moveto 90 rotate", 0, -extent_y, "translate"
				}
				else
				{
					# even page
					print "0 0 moveto -90 rotate", -extent_x, 0, "translate"
				}
			}
			print ""
		}
	}

	# spine text
	if (format == "<st>")
	{
		print "% spine text"
		print "/spine_text", text, "def"
	}
}

function new_page(format)
{
	# new page for table, unnumbered & numbered pages, and for tumble blank page if tumble enabled
	if ((format == "<tb>") || (format == "<p0>") || (format == "<p1>") || ((format == "<pt>") && (tumble == 1)))
	{
		# check for first page (end of prolog)
		if (page == 0)
			close_prolog()

		# Finish previous page
		# undo table offset, before we try to print a page number
		if ((page > 0) && (page_format == "<tb>"))
		{
			print "%	undo offset to bottom left of table"
			if ((page % 2) == 1)
			{
				# odd page
				if (orientation == "landscape")
					print 0, extent_y, "translate 0 0 moveto -90 rotate"
			}
			else
			{
				# even page
				if (orientation == "landscape")
					print -extent_x, 0, "translate 0 0 moveto 90 rotate"
			}
			print -binding_margin, -footer_margin, "translate"
		}

		# finish page features
		if (sheet > 0)
		{
			# spine & binding
			print "% spine & binding"
			# print spine unless blank tumble page
			if ((spine == 1) && (format != "<pt>"))
			{
				print "paper spine"
				print exped_tab_x, "expedTab"
			}

			# if enabled, print binding holes every 8 sheets
			if (((sheet % 8) == 1) && (do_binding == 1))
			{
				print "paper binding"
			}
		}

		# check if page number printing is pending, and this is a printing page
		if (pn_pend == 1)
		{
			# only print page numbers for section > 0
			if (sect_num > 0)
			{
				print "% page number"
				print "select_p1_font"
				print p1_x+wide_page, p1_y, "moveto"
				# reset wide page
				wide_page	= 0
				# select page numbering style
				if (num_style == 1)
					printf("(%d%s%d) centreshow\n", sect_num, num_sep, sect_page)
				else
					printf("(%d) centreshow\n", number)
			}
			# clear pending page number
			pn_pend	= 0
		}
		# check if we need to clear number pending flag
		if ((format == "<p1>") || (format == "<tb>"))
			pn_pend	= 1
		else
			pn_pend	= 0

		# undo tumbling & Q7 binding offset
		if (page > 0)
		{
			# undo offset & rotation for tumbled even page
			if (((sheet % 2) == 0) && (tumble == 1))
			{
				print "% undo tumble binding offset"
				print 0, -tumble_y, "translate"
				print "% untumble"
				print "0 0 moveto"
				print "180 rotate"
				print -page_x, -page_y, "translate"
			}
			# undo offset for Q7 page binding
			if ((paper == "Q7") && ((sheet % 2) == 1))
			{
				print "% undo Q7 binding offset"
				print -Q7_bind, 0, "translate"
			}

			print ""
			print "showpage"
			print ""
		}


		# Start new sheet (PostScript page)
		# increment physical and numbered page counts
		page++
		if ((format == "<p1>") || (format == "<tb>"))
		{
			number++
			sect_page++
		}

		# increment sheet count
		sheet++

		# reset rotated page flag
		rot_page = 0

		# note the page type
		page_format	= format

		# print sheet header
		print ""
		print "%%Page:", sheet, sheet
		print "%%BeginPageSetup"
		print "% convert to mm scaling"
		print "72 25.4 div dup scale"
		# add printer offset correction on even sheets
		if ((sheet % 2) == 1)
		{
			print "% printer registration correction"
			print "oddCorrect"
		}
		else
		{
			print "% printer registration correction"
			print "evenCorrect"
		}

		print "%%EndPageSetup"

		# set initial place in page
		pos_x		= margin_x
		pos_y		= page_y-margin_top
		exped_tab_x	= page_x

		# setup page 
		{
			# document page
			print "% document page", page

			# position on sheet
			print "% page position adjustments"
			# tumbled binding
			if (tumble == 1)
			{
				if ((sheet % 2) == 1)
					# move odd page content down to clear sewn binding
					pos_y	= pos_y - tumble_y
				else
				{
					# rotate even page
					print "% tumble"
					print "0 0 moveto"
					print "180 rotate"
					print -page_x, -page_y, "translate"
					# offset entire page to clear sewn binding
					print "% tumble binding offset"
					print 0, tumble_y, "translate"
				}
			}

			# offset for Q7 page binding
			if ((paper == "Q7") && ((sheet % 2) == 1))
			{
				print "% Q7 binding offset"
				print Q7_bind, 0, "translate"
			}

			# colour
			print "% colour"
			print body_colour
		}
	}		# end new page
}

function close_prolog()
{
	# write derived values
	print ""
	print "% derived values"

	print "%%EndProlog"
	print ""

	# select PostScript document tumble printing mode
	print "%%BeginSetup"
	print "<< /Duplex true /Tumble false >> setpagedevice"
	print ""

	print "%%EndSetup"
}

function table_features(format, text)
{
	# page outline
	if (format == "<po>")
	{
		# draw page outline
		print "% draw page outline"
		print "gsave"
		# check for line thickness parameter
		if (NF == 2)
			print $2 / paper_scale, "setlinewidth"
		# set line end and join styles
		print "2 setlinecap 0 setlinejoin [] 0 setdash"
		print "0 0 moveto"
		print extent_x, 0, "lineto"
		print extent_x, extent_y, "lineto"
		print 0, extent_y, "lineto"
		print "closepath stroke"
		print "grestore"
		print ""
	}

	# table area definition, as % of table extent
	#	expects definition file to be sensibly hierarchical
	if (format == "<ad>")
	{
		area++
		print "% area definition", area
		values[area, "area_x"]			= $2 * extent_x / 100
		values[area, "area_y"]			= $3 * extent_y / 100
		print "%	area", area
		print "%	area size", values[area, "area_x"], values[area, "area_y"]
		values[area, "pos_x"]			= $4 * extent_x / 100
		values[area, "pos_y"]			= $5 * extent_y / 100
		print "%	area position", values[area, "pos_x"], values[area, "pos_y"]
		# initialise column definitions
		cols	= 0
		values[area, 0, "col_pos_x"]	= 0		# left
		values[area, 1, "col_pos_x"]	= values[area, "area_x"]	# right
		print ""
	}

	# table area outline
	if (format == "<ao>")
	{
		# draw area outline
		print "% draw area outline", area
		print "gsave"
		# check for line thickness parameter
		if (NF == 2)
			print $2 / paper_scale, "setlinewidth"
		# set line end and join styles
		print "2 setlinecap 0 setlinejoin [] 0 setdash"
		print values[area, "pos_x"], values[area, "pos_y"], "moveto"
		print values[area, "area_x"], 0, "rlineto"
		print 0, values[area, "area_y"], "rlineto"
		print -values[area, "area_x"], 0, "rlineto"
		print "closepath stroke"
		print "grestore"
		print ""
	}

	# define number of rows in this area
	if (format == "<rd>")
	{
		print "% row definition", $2
		values[area, "rows"]	= $2
		# derived values
		values[area, "row_y"]	= values[area, "area_y"] / $2
		print "%	rows", values[area, "rows"]
		print "%	row_y", values[area, "row_y"]
		print ""
	}

	# draw row lines
	#	<rl> min_r max_r min_col max_col
	if (format == "<rl>")
	{
		print "% draw row lines"
		# check not exceeding row & col counts (allow 1 over for drawing final row)
		if (($2-1 <= values[area, "rows"]) && ($3-1 <= values[area, "rows"]) && ($4-1 <= values[area, "cols"]) && ($5-1 <= values[area, "cols"]))
		{
			# get values
			#	x start position
			# left side of first column (right side of previous column)
			start_x	= values[area, "pos_x"] + values[area, $4-1, "col_pos_x"]

			#	x stop position
			# right side of last column
			stop_x	= values[area, "pos_x"] + values[area, $5, "col_pos_x"]

			#	y start position
			start_y	= values[area, "pos_y"] + values[area, "area_y"] - ($2-1) * values[area, "row_y"]

			# for each row
			for (row = $2-1; row < $3; row++)
			{
				print "% row", row
				print start_x, start_y, "moveto"
				print stop_x, start_y, "lineto"
				start_y	= start_y - values[area, "row_y"]
			}
			print "stroke"
		}
		else
			print FILENAME, "row line: too many rows", $0 | "cat 1>&2"
		print ""
	}

	# column definitions (assumes all columns defined L-R, in order)
	#	horizontal position % of row_width
	if (format == "<cd>")
	{
		print "% column definitions"
		# record each column
		for (field = 2; field <= NF; field++)
		{
			the_col		= field - 1
			# record number of columns & this column horizontal right margin
			values[area, the_col, "col_pos_x"]	= values[area, "area_x"] * $field / 100
			print "%	col_pos_x", the_col, values[area, the_col, "col_pos_x"]
		}
		# ensure last column is 100%, and record number of columns
		if ($NF != 100)
		{
			values[area, NF, "col_pos_x"]	= values[area, "area_x"]
			print "%	col_pos_x", NF, values[area, NF, "col_pos_x"]
			values[area, "cols"]			= NF
		}
		else
			values[area, "cols"]			= NF-1
		print ""
	}

	# column lines (assumes all columns defined by <cd>)
	#	horizontal position % of col, start_row, end_row [start_row_n, end_row_n]
	#	rows defined top down
	if (format == "<cl>")
	{
		the_col		= $2
		# check column defined
		if (the_col <= values[area, "cols"])
		{
			print "% draw column", the_col, "lines"
			# get start & stop pairs
			for (param = 3; param < NF; param=param+2)
			{
				# check not exceeding row counts
				if (($param <= values[area, "rows"]) && ($(param+1) <= values[area, "rows"]))
				{
					# draw lines
					#	col x, top of start row in area 
					print values[area, "pos_x"] + values[area, the_col, "col_pos_x"], values[area, "pos_y"] + values[area, "area_y"] - ($param-1) * (values[area, "row_y"]), "moveto"
					#	vertical line, length of number of rows
					print 0, ($(param) - $(param+1) - 1) * values[area, "row_y"], "rlineto stroke"
				}
				else
					print "column: too many rows", $0 | "cat 1>&2"
			}
			print ""
		}
		else
			print "column not defined:", $0 | "cat 1>&2"
	}

	# border thickness (absolute (mm))
	if (format == "<bt>")
	{
		print "% border line thickness"
		print $2/paper_scale, "setlinewidth"
		print ""
	}

	# border format (colour)
	if (format == "<bf>")
	{
		print "% border line colour"
		print $2
	}

	# border line style (cap, join, dash)
	if (format == "<bs>")
	{
		print "% border line style"
		print $2
		print ""
	}
	
	# cell colour fill
	#	<fi> row col <colourname|postscript colour definition>
	#	<fi> min_col max_col min_row max_row <colourname|postscript colour definition>
	if (format == "<fi>")
	{
		print "% colour fill"
		print "gsave"
		# multiple cells?
		if (NF == 6)
		{
			# check bounds valid
			if (($3 > values[area, "cols"]) || ($5 > values[area, "rows"]))
				print "fill area bounds invalid:", values[area, "rows"], values[area, "cols"], $0 | "cat 1>&2"
			else
			{
				print "% multi-cell fill"
				start_x		= values[area, "pos_x"] + values[area, $3, "col_pos_x"]
				start_y		= values[area, "pos_y"] + values[area, "area_y"] - $5 * values[area, "row_y"]
				delta_x		= values[area, $2-1, "col_pos_x"] - values[area, $3, "col_pos_x"]
				delta_y		= values[area, "row_y"] * ($5-$4+1)
				colour		= $6
			}
		}
		else
		{
			print "% single cell fill"
			start_x		= values[area, "pos_x"] + values[area, $2, "col_pos_x"]
			start_y		= values[area, "pos_y"] + values[area, "area_y"] - $3 * values[area, "row_y"]
			delta_x		= values[area, $3-1, "col_pos_x"] - values[area, $2, "col_pos_x"]
			delta_y		= values[area, "row_y"]
			colour		= $4
		}
		# draw box
		print start_x, start_y, "moveto"
		print delta_x, 0, "rlineto"
		print 0, delta_y, "rlineto"
		print -delta_x, 0, "rlineto"
		# close path, set colour & fill
		print "closepath"
		print colour
		print "fill"
		print "grestore"
		print ""
	}

	# text function
	# font definitions
	#	font face
	if (format == "<ft>")
	{
		print "% font name"
		values[area, "font"]	= $2
	}
	# fixed font size & offsets (mm)
	if (format == "<ff>")
	{
		print "% font size % offsets (absolute)"
		values[area, "font_size"]	= $2
		values[area, "font_xoff"]	= $3
		values[area, "font_yoff"]	= $4
	}
	# relative font size & offsets (% of row height)
	# <fr> size% xoff% yoff%
	if (format == "<fr>")
	{
		print "% font size % offsets (relative)"
		# derived values
		values[area, "font_size"]	= $2 * values[area, "row_y"] / 100
		values[area, "font_xoff"]	= $3 * values[area, "row_y"] / 100
		values[area, "font_yoff"]	= $4 * values[area, "row_y"] / 100
	}
	# activate the font as defined above
	if (format == "<fa>")
	{
		print "% activate font"
		print values[area, "font"], values[area, "font_size"], "selectfont"
		print ""
	}

	# print text string in selected row, column position
	#	<tx>	[<](string)[>] row col1 [col2]
	#	<(string)	left justified
	#	<(string)>	centred
	#	(string)>	right justified
	#	if there are two col parameters, will position between the two
	#		use for centre justified
	if (format == "<tx>")
	{
		# top of area - rows + text offset
		text_y	= (values[area, "pos_y"] + values[area, "area_y"]) - ($3 * values[area, "row_y"]) + values[area, "font_yoff"]

		# horizontal position
		left_x	= values[area, "pos_x"] + values[area, $4-1, "col_pos_x"]
		if (NF == 5)
			right_x	= values[area, "pos_x"] + values[area, $5, "col_pos_x"]
		else
			right_x	= values[area, "pos_x"] + values[area, $4, "col_pos_x"]

		print "% string"

		# left justified
		if ((substr($2, 1, 1) == "<") && (substr($2, length($2), 1) != ">"))
		{
			text_x	= left_x + values[area, "font_xoff"]
			print text_x, text_y, "moveto"
			print substr($2, 2), "show"
		}

		# centre justified
		if ((substr($2, 1, 1) == "<") && (substr($2, length($2), 1) == ">") )
		{
			text_x	= (left_x + right_x) / 2
			print text_x, text_y, "moveto"
			print substr($2, 2, length($2)-2), "centreshow"
		}

		# right justified
		if ((substr($2, 1, 1) != "<") && (substr($2, length($2), 1) == ">")) 
		{
			text_x	= right_x - values[area, "font_xoff"]
			print text_x, text_y, "moveto"
			print substr($2, 1, length($2)-1), "rightshow"
		}

		# title
		if (substr($2, 1, 1) == ">") 
		{
			if ((page % 2) == 1)
			{
				# odd page
				print "% odd page title flush right"
#				print "% extent_x extent_y", extent_x, extent_y
				text_x	= extent_x - values[area, "font_xoff"]
				text_y	= extent_y + values[area, "font_yoff"]
				print text_x, text_y, "moveto"
				print substr($2, 2, length($2)-1), "rightshow"
			}
			else
			{
				# even page
				print "% even page title flush left"
				print "% extent_x extent_y", extent_x, extent_y
				text_x	= values[area, "font_xoff"]
				text_y	= extent_y + values[area, "font_yoff"]
				print text_x, text_y, "moveto"
				print substr($2, 2, length($2)-1), "show"
			}
		}
		print ""
	}

	# Area fill functions
	#
	# fill area with bujo
	#	row_y spacing in x and y
	#	row_y starting position for x & y
	#	max_y top of area, less half row_y, to prevent drawing dot grid on margin
	if (format == "<bj>")
	{
		if (NF == 1)
		{
			# fill entire area with bujo grid
			#	min_y max_y min_x max_x step
			print "% fill area with bujo grid, at row_y spacing"
			print values[area, "pos_y"] + values[area, "row_y"], values[area, "pos_y"] + values[area, "area_y"]-values[area, "row_y"]/2, values[area, "pos_x"] + values[area, "row_y"], values[area, "pos_x"] + values[area, "area_x"], values[area, "row_y"], "bujo"
			print ""
		}
		else
		{
			# fill specific area with bujo
			print "% fill specific area with bujo grid, at row_y spacing"
print "bujo: oops - sub-area fill not yet implemented, line", NR, $0 | "cat 1>&2"
			print ""
		}
	}

	# graphic: EPS
	# <te> size_x size_y filename
	# should be last element in a page
	if (format == "<te>")
	{
		print "% insert EPS image:", $4
		# get image dimensions
		pic_x	= $2
		pic_y	= $3
		print "%	dimensions:", pic_x, pic_y

		# available space
		avail_x	= values[area, "area_x"]
		avail_y	= values[area, "area_y"]

		# fit image into available space
		#	between page left/right margins
		scale_x	= avail_x / pic_x
		#	between bottom of text & bottom of page
		scale_y	= avail_y / pic_y

		# choose smaller scale & place graphic
		if (scale_x < scale_y)
		{
			# fill in x dimension
			clip_x			= scale_x * pic_x
			clip_y			= scale_x * pic_y
			# no gap in x dimension
			pic_x_offset	= 0
			# add gap in y dimension
			pic_y_offset	= (avail_y - clip_y)/2
			the_scale		= scale_x
		}
		else
		{
			# fill in y dimension
			clip_x			= scale_y * pic_x
			clip_y			= scale_y * pic_y
			# add gap in x dimension
			pic_x_offset	= (avail_x - clip_x)/2
			# no gap in y dimension
			pic_y_offset	= 0
			the_scale		= scale_y
		}

		# set scale and origin
		print "%	available image space:", avail_x, avail_y
		print "%	scaled image space:", clip_x, clip_y
		print "%	save current scale & origin"
		print "gsave"
		print "%	translate to bottom left corner of image"
		print values[area, "pos_x"] + pic_x_offset, values[area, "pos_y"] + pic_y_offset, "translate"
		print "%	scale to fit larger dimension in available space"
		print the_scale, the_scale, "scale"

		# blank background
		print "%	blank image background"
		print "white"
		print "newpath"
		print "0 0 moveto"
		print pic_x, 0, "rlineto"
		print 0, pic_y, "rlineto"
		print -pic_x, 0, "rlineto"
		print "closepath"
		print "fill"
		print "black"

		# insert image
		print "%	insert image"
		printf("(%s) run\n", $4)

		print "%	restore page origin & scale"
		print "grestore"
		print ""

		# detect any printable lines after graphic
		lines	= max_lines
	}

	# graphic: JPEG
	# <tj> size_x size_y filename
	if (format == "<tj>")
	{
		print "% insert JPEG image:", $4
		# get image dimensions
		pic_x	= $2
		pic_y	= $3
		print "%	dimensions:", pic_x, pic_y
		# available space
		avail_x	= values[area, "area_x"]
		avail_y	= values[area, "area_y"]
		print "%	available image space:", avail_x, avail_y
		# fit image into available space
		# between page left/right margins
		scale_x	= avail_x / pic_x
		# between bottom of text & bottom of page
		scale_y	= avail_y / pic_y

		# choose smaller scale & calculate scaled image dimensions
		if (scale_x < scale_y)
		{
			# fill in x dimension
			clip_x			= scale_x * pic_x
			clip_y			= scale_x * pic_y
			# no gap in x dimension
			pic_x_offset	= 0
			# add gap in y dimension
			pic_y_offset	= (avail_y - clip_y)/2
			the_scale		= scale_x
		}
		else
		{
			# fill in y dimension
			clip_x			= scale_y * pic_x
			clip_y			= scale_y * pic_y
			# add gap in x dimension
			pic_x_offset	= (avail_x - clip_x)/2
			# no gap in y dimension
			pic_y_offset	= 0
			the_scale		= scale_y
		}

		print "%	scaled image space:", clip_x, clip_y
		print "%	save current scale & origin"
		print "gsave"
		print "%	translate to bottom left corner of image"
		print values[area, "pos_x"] + pic_x_offset, values[area, "pos_y"] + pic_y_offset, "translate"
		print "%	scale to fit larger dimension in available space"
		print the_scale, the_scale, "scale"

		# scale to unit image
		print "%	scale to make unit image fit"
		print pic_x, pic_y, "scale"

		# print image
		print "%	read image into unit square"
		print "%	parameters for colorimage"
		print pic_x, "					% width"
		print pic_y, "					% height"
		print 8, "						% 8 bits per color channel"
		print "%	[width 0 0 -height 0 height] scan L-R, B-T"
		print "[" pic_x, 0, 0, -pic_y, 0, pic_y "] % matrix		transform array... maps unit square to pixel"
		if (embed == 1)
		{
			print "%	embed JPEG as string"
			printf("(")
			system("hexdump -v -e'32/1 \"%02x\" \"\n\"' " $4)
#			system("hexdump -v -e'/1 \"%02x\"' " $4)
			printf(")\n")
			print "%	convert ASCII to bytes"
			print "/ASCIIHexDecode filter"
		}
		else
		{
			print "(" $4 ") (r) file		% open the file"
		}
		print "%	decode DCT to image"
		print "/DCTDecode filter		% filter the image data"
		print "false					% multi			pull channels from separate sources)"
		print "3						% ncomp			3 color channels (RGB)"
		print "colorimage				% width height bits/comp matrix <datasrc0> multi ncomp - "

		print "%	restore page origin & scale"
		print "grestore"
		print ""
	}
}

function section_page(format)
{
	# parse Chapter format first, to ensure all Sections are counted
	if ((format == "<c0>") || (format == "<s0>"))
	{
		# we want Chapter or Section to start on odd page (sect_even == 1);
		# easier to insert corrections, or add pages to sections
		# if page count is odd, insert blank page
		if ((sect_even == 1) && ((page % 2) == 0))
		{
			new_page(page_format)
			# reset line counter
			lines	= 0
		}
		if (format == "<c0>")
			page_format	= "<p0>"
		if (format == "<s0>")
		{
			# start new Section
			sect_num++
			sect_page	= 1
		}
	}
}

function cover_features(format)
{
	# enable binding holes (unless book)
	if ((format == "<bh>") && (book == 0))
	{
		# enable binding printing on subsequent sheets
		do_binding	= 1
	}

	# logo
	if (format == "<l0>")
	{
		print "% visible logo"
		print dofe_colour
		offset_y	= l0_gap		# move to logo position
		# normal logo
		if (book == 1)
			print tabWidth+page_x/20, stripeStart+tabHeight-l0_height, "moveto				% offset logo at spine width + 5% page width"
		else
		{
			# logo offset from horizontal centre by half scaled width, and from top of text area
			print t_horiz - (l0_kern * l0_height/l0_font)/2, pos_y-offset_y, "moveto	% bottom left of logo"
		}
		print "% draw logo at desired height"
		print l0_height, "logo"

		# add shiny logo to book page
		if (book == 1)
		{
			print "% shiny print logo"
			print "shiny"
			print "26 46 moveto"
			print "page_x logo"
		}
		print body_colour
	}

	# title 0
	if (format == "<t0>")
	{
		if (format != last_format)
		{
			if (book == 1)
			{
			print "% document title box"
			print "0 setlinewidth"
			print "brick"
			print tabWidth, tabStart-tabSpace-tabHeight, "moveto"
			print "0", 2*tabSpace+3*tabHeight, "rlineto"
			print "100 0 rlineto"
			print "0", -2*tabSpace-3*tabHeight, "rlineto"
			print "-100 0 rlineto"
			print "fill"
			print "white"
			}
			print book, "select_t0_font"
		}
		# title line not following from logo or previous title line?
		if (((book == 1) || (last_format != "<l0>")) && (last_format != "<t0>"))
			offset_y	= t0_gap		# move to title0 position
		else
			offset_y	+= t0_space		# advance down page
		if (book == 1)
		{
			print tabWidth+page_x/20, offset_y, "moveto"
			printf("(%s) show\n", text)
		}
		else
		{
			print pos_x+t_horiz-margin_x, pos_y-offset_y, "moveto"
			printf("(%s) centreshow\n", text)
		}
	}

	# title 1
	if (format == "<t1>")
	{
		if (book == 1)
		{
			print book, "select_t1_font"
			print "brick"
			print 130, offset_y, "moveto"
			printf("(%s %s) show\n", text, file_version)
		}
		else
		{
			if (format != last_format)
			{
				print book, "select_t1_font"
				offset_y	= t1_gap		# move to title1 position
			}
			else
				offset_y	+= t1_space		# advance down page
			print pos_x+t_horiz-margin_x, pos_y-offset_y, "moveto"
			# print filedate if requested
			if (index(text, "<date>") != 0)
				{
				text = substr(text, 1, length(text)-6)
				printf("(%s%s) centreshow\n", text, filedate)
				}
			else
				printf("(%s) centreshow\n", text)
		}
	}

	# title 2
	if ((format == "<t2>") && (book == 0))
	{
		if (format != last_format)
		{
			print book, "select_t2_font"
			offset_y	= t2_gap		# move to title2 position
		}
		else
			offset_y	+= t2_space		# advance down page
		print pos_x+t_horiz-margin_x, pos_y-offset_y, "moveto"
		printf("(%s) centreshow\n", text)
	}
}

function history_features(format)
{
	# file history
	if ((format == "<fh>") && (book == 0))
	{
	print "% file history"
		if ((last_format != "<fh>") && (last_format != "<t1>"))
		{
			print book, "select_t1_font"
			offset_y	= t1_gap		# move to title1 position
		}
		else
			offset_y	+= t1_space		# advance down page
		print pos_x+t_horiz-margin_x, pos_y-offset_y, "moveto"
		printf("(%s %s) centreshow\n", file_version, filedate)
	}

	# print history
	if (format == "<ph>")
	{
	print "% print history"
		if (format != last_format)
			print "select_ph_font"
		print pos_x+t_horiz-margin_x, ph_posn, "moveto"
		printf("(created from %s %s %s by %s %s %s at %s) centreshow\n", source_file, file_version, filedate, script_file, script_version, scriptdate, time)
	}
}

function footnote(format, text)
{
	# footnote
	if (format == "<fn>")
	{
	print "% footnote"
		if (format != last_format)
			print "select_ph_font"
		print pos_x, ph_posn, "moveto"
		printf("(%s) show\n", text)
	}
}

function printed_lines(format, text)
{
	# count printed lines
	#	reset line count at start of page
	if ((format == "<p0>") || (format == "<p1>"))
	{
		# reset printed line counter
		lines		= 0
	}
	#	increment line counter for each printable line
	if ((format != "") && (index(print_lines, format) != 0))
	{
		# increment line counter
		lines	= lines + 1

		# report lines per page exceeded to std_err
		if (lines == max_lines+1)
		{
			print "more than", max_lines, "lines per page" | "cat 1>&2"
			print "page:", page | "cat 1>&2"
		}
	}

	# chapter
	if (format == "<c0>")
	{
		chap_num++
		print "% chapter"
		if (format != last_format)
			print "select_c0_font"
		pos_y -= c0_space
		print pos_x, pos_y, "moveto"
		printf("(CHAPTER %d: %s) show\n", chap_num, text)
	}

	# section
	if (format == "<s0>")
	{
		print "% section"
		if (format != last_format)
			print "select_s0_font"
		pos_y -= s0_space
		print pos_x, pos_y, "moveto"
		printf("(Section %d: %s) show\n", sect_num, text)
	}

	# header 0
	if (format == "<h0>")
	{
		print "% heading"
		if (format != last_format)
			print "select_h0_font"
		pos_y -= h0_space
		print pos_x, pos_y, "moveto"
		printf("(%s) show\n", text)
	}
	# bullet 0
	if (format == "<b0>")
	{
		if (format != last_format)
			print "select_b0_font"
		pos_y -= b0_space
		print pos_x, pos_y, "moveto"
		printf("(%s) show\n", text)
	}
	# italic 0
	if (format == "<i0>")
	{
		if (format != last_format)
			print "select_i0_font"
		pos_y -= b0_space
		print pos_x, pos_y, "moveto"
		printf("(%s) show\n", text)
	}
	# tickbox 0
	if (format == "<k0>")
	{
		if (format != last_format)
			print "select_h0_font"
		pos_y -= h0_space
		print pos_x, pos_y, "moveto"
		printf("(%s) show\n", text)
		print "tickBox0"
	}

	# header 1
	if (format == "<h1>")
	{
		print "% heading"
		if (format != last_format)
			print "select_h1_font"
		pos_y -= h1_space
		print pos_x, pos_y, "moveto"
		print tab_x, 0, "rmoveto"
		printf("(%s) show\n", text)
	}
	# bullet 1
	if (format == "<b1>")
	{
		if (format != last_format)
			print "select_b1_font"
		pos_y -= b1_space
		print pos_x, pos_y, "moveto"
		print tab_x, 0, "rmoveto"
		printf("(%s) show\n", text)
	}
	# italic 1
	if (format == "<i1>")
	{
		if (format != last_format)
			print "select_i1_font"
		pos_y -= b1_space
		print pos_x, pos_y, "moveto"
		print tab_x, 0, "rmoveto"
		printf("(%s) show\n", text)
	}
	# tickbox 1
	if (format == "<k1>")
	{
		if (format != last_format)
			print "select_b1_font"
		pos_y -= b1_space
		print pos_x, pos_y, "moveto"
		print tab_x, 0, "rmoveto"
		printf("(%s) show\n", text)
		print "tickBox1"
	}

	# header 2
	if (format == "<h2>")
	{
		if (format != last_format)
			print "select_h2_font"
		pos_y -= h2_space
		print pos_x, pos_y, "moveto"
		print 2*tab_x, 0, "rmoveto"
		printf("(%s) show\n", text)
	}
	# bullet 2
	if (format == "<b2>")
	{
		if (format != last_format)
			print "select_b2_font"
		pos_y -= b2_space
		print pos_x, pos_y, "moveto"
		print 2*tab_x, 0, "rmoveto"
		printf("(%s) show\n", text)
	}
	# italic 2
	if (format == "<i2>")
	{
		if (format != last_format)
			print "select_i2_font"
		pos_y -= b2_space
		print pos_x, pos_y, "moveto"
		print 2*tab_x, 0, "rmoveto"
		printf("(%s) show\n", text)
	}

	# contact details
	if (format == "<c1>")
	{
		if (format != last_format)
		{
			print "select_c1_font"
			print contStroke, "setlinewidth"
			print "[2] 0 setdash"
		}
		pos_y -= b1_space
		print pos_x, pos_y, "moveto"
		print tab_x, 0, "rmoveto"
		printf("(%s) show\n", $2)
		print pos_x, pos_y, "moveto"
		print 5*tab_x, 0, "rmoveto"
		if (NF > 2)
			printf("(%s %s %s) show\n", $3, $4, $5)
		else
		{
			print contWidth, 0, "rlineto"
			print "stroke"
		}
	}

	# colour
	# see 'colours.ps' for available colours
	if (format == "<co>")
		print text

	# graphic: EPS
	# ge size_x size_y filename
	# should be last element in a page
	if (format == "<ge>")
	{
		print "% insert EPS image:", $4
		# add a small gap below text
		pos_y -= t2_space
		# get image dimensions
		pic_x	= $2
		pic_y	= $3

		# available space
		avail_x	= page_x - 2*margin_x
		avail_y	= pos_y - image_y

		# set scale and origin
		print "%	save page origin & scale"
		print "gsave"

		# fit image into available space
		#	between page left/right margins
		scale_x	= avail_x / pic_x
		#	between bottom of text & bottom of page
		scale_y	= avail_y / pic_y

		# choose smaller scale & calculate scaled image dimensions
		if (scale_x < scale_y)
		{
			clip_x			= scale_x * pic_x
			clip_y			= scale_x * pic_y
			pic_x_offset	= 0
			the_scale		= scale_x
		}
		else
		{
			clip_x			= scale_y * pic_x
			clip_y			= scale_y * pic_y
			pic_x_offset	= (avail_x - clip_x)/2
			the_scale		= scale_y
		}

		print "%	set image origin & scale"
		print margin_x + pic_x_offset, image_y, "translate"
		print "%	dimensions:", pic_x, pic_y
		print "%	available image space:", avail_x, avail_y
		print "%	used image space:", clip_x, clip_y
		print the_scale, the_scale, "scale"

		# blank background
		print "%	blank image background"
		print "white"
		print "newpath"
		print "0 0 moveto"
		print pic_x, 0, "rlineto"
		print 0, pic_y, "rlineto"
		print -pic_x, 0, "rlineto"
		print "closepath"
		print "fill"
		print "black"

		# insert image
		print "%	insert image"
		printf("(%s) run\n", $4)

		print "%	restore page origin & scale"
		print "grestore"

		# detect any printable lines after graphic
		lines	= max_lines
	}

	# graphic: EPS - spread over three page widths
	# gEx size_x size_y filename
	# should be last element in a page
	if (format == "<gE3>")
	{
		print "% insert 3-page EPS image:", format, $4
		# add a small gap below text
		pos_y -= t2_space
		# get image dimensions
		pic_x	= $2
		pic_y	= $3

		# available space
		avail_x	= 3*page_x - 4*margin_x		# triple page width: 1 * page_x and 2 * (page_x-margin_x)
		avail_y	= pos_y - image_y

		# set scale and origin
		print "%	save page scale & origin"
		print "gsave"

		# fit image into available space
		#	between page left/right margins
		scale_x	= avail_x / pic_x
		#	between bottom of text & bottom of page
		scale_y	= avail_y / pic_y

		# choose smaller scale & calculate scaled image dimensions
		if (scale_x < scale_y)
		{
			clip_x			= scale_x * pic_x
			clip_y			= scale_x * pic_y
			pic_x_offset	= 0
			the_scale		= scale_x
		}
		else
		{
			clip_x			= scale_y * pic_x
			clip_y			= scale_y * pic_y
			pic_x_offset	= (avail_x - clip_x)/2
			the_scale		= scale_y
		}

		print "%	set image origin & scale"
		print margin_x + pic_x_offset, image_y, "translate"
		print "%	dimensions:", pic_x, pic_y
		print "%	available image space:", avail_x, avail_y
		print "%	used image space:", clip_x, clip_y
		print the_scale, the_scale, "scale"

		# blank background
		print "%	blank image background"
		print "white"
		print "newpath"
		print "0 0 moveto"
		print pic_x, 0, "rlineto"
		print 0, pic_y, "rlineto"
		print -pic_x, 0, "rlineto"
		print "closepath"
		print "fill"
		print "text"

		# insert image
		print "%	insert scaled image"
		printf("(%s) run\n", $4)

		print "%	restore page origin & scale"
		print "grestore"

		# print exped tab at edge of folded page
		exped_tab_x	= avail_x + 2*margin_x

		# adjust page number position
		wide_page	= 2*(page_x-margin_x)

		# detect any printable lines after graphic
		lines		= max_lines
	}

	# graphic: JPEG
	# gj size_x size_y filename
	# should be last element in a page
	if (format == "<gj>")
	{
		print "% insert JPEG image:", $4
		# add a small gap below text
		pos_y -= t2_space
		# get image dimensions
		pic_x	= $2
		pic_y	= $3

		# available space
		avail_x	= page_x - 2*margin_x
		avail_y	= pos_y - image_y

		# set scale and origin
		print "%	save current scale & origin"
		print "gsave"

		# fit image into available space
		#	between page left/right margins
		scale_x	= avail_x / pic_x
		#	between bottom of text & bottom of page
		scale_y	= avail_y / pic_y

		# choose smaller scale & calculate scaled image dimensions
		if (scale_x < scale_y)
		{
			clip_x			= scale_x * pic_x
			clip_y			= scale_x * pic_y
			pic_x_offset	= 0
			the_scale		= scale_x
		}
		else
		{
			clip_x			= scale_y * pic_x
			clip_y			= scale_y * pic_y
			pic_x_offset	= (avail_x - clip_x)/2
			the_scale		= scale_y
		}

		print "%	set image origin & scale"
		print margin_x + pic_x_offset, image_y, "translate"
		print "%	dimensions:", pic_x, pic_y
		print "%	available image space:", avail_x, avail_y
		print "%	used image space:", clip_x, clip_y
		print the_scale, the_scale, "scale"

		# scale to unit image
		print "%	scale to make unit image fit"
		print pic_x, pic_y, "scale"

		# print image
		print "%	read image into unit square"
		print "%	parameters for colorimage"
		print pic_x, "					% width"
		print pic_y, "					% height"
		print 8, "						% 8 bits per color channel"
		print "%	[width 0 0 -height 0 height] scan L-R, B-T"
		print "[" pic_x, 0, 0, -pic_y, 0, pic_y "] % matrix		transform array... maps unit square to pixel"
		if (embed == 1)
		{
			print "%	embed JPEG as DSC-compliant strings"
			print "("
			system("hexdump -v -e '64/1 \"%02x\" \"\n\"' " $4)
			print ")"
			print "%	convert ASCII to bytes"
			print "/ASCIIHexDecode filter"
		}
		else
		{
			print "(" $4 ") (r) file		% open the file"
		}
		print "%	decode DCT to image"
		print "/DCTDecode filter		% filter the image data"
		print "false					% multi			pull channels from separate sources)"
		print "3						% ncomp			3 color channels (RGB)"
		print "colorimage				% width height bits/comp matrix <datasrc0> multi ncomp - "

		print "%	restore page origin & scale"
		print "grestore"

		# detect any printable lines after graphic
		lines	= max_lines
	}

	# graphic: JPEG - spread over three page widths
	# gJx size_x size_y filename
	# should be last element in a page
	if (format == "<gJ3>")
	{
		print "% insert 3-page JPEG image:", format, $4
		# add a small gap below text
		pos_y -= t2_space
		# get image dimensions
		pic_x	= $2
		pic_y	= $3

		# available space
		avail_x	= 3*page_x - 4*margin_x		# triple page width: 1 * page_x and 2 * (page_x-margin_x)
		avail_y	= pos_y - image_y

		# set scale and origin
		print "%	save current scale & origin"
		print "gsave"

		# fit image into available space
		# between page left/right margins
		scale_x	= avail_x / pic_x
		# between bottom of text & bottom of page
		scale_y	= avail_y / pic_y

		# choose smaller scale & calculate scaled image dimensions
		if (scale_x < scale_y)
		{
			clip_x			= scale_x * pic_x
			clip_y			= scale_x * pic_y
			pic_x_offset	= 0
			the_scale		= scale_x
		}
		else
		{
			clip_x			= scale_y * pic_x
			clip_y			= scale_y * pic_y
			pic_x_offset	= (avail_x - clip_x)/2
			the_scale		= scale_y
		}

		print "%	set image origin & scale"
		print margin_x, image_y, "translate"
		print "%	dimensions:", pic_x, pic_y
		print "%	available image space:", avail_x, avail_y
		print "%	used image space:", clip_x, clip_y
		print the_scale, the_scale, "scale"

		# scale to unit image
		print "%	scale to make unit image fit"
		print pic_x, pic_y, "scale"

		# print image
		print "%	read image into unit square"
		print "%	parameters for colorimage"
		print pic_x, "					% width"
		print pic_y, "					% height"
		print 8, "						% 8 bits per color channel"
		print "%	[width 0 0 -height 0 height] scan L-R, B-T"
		print "[" pic_x, 0, 0, -pic_y, 0, pic_y "] % matrix		transform array... maps unit square to pixel"
		if (embed == 1)
		{
			print "%	embed JPEG as DSC-compliant strings"
			print "("
			system("hexdump -v -e '64/1 \"%02x\" \"\n\"' " $4)
			print ")"
			print "%	convert ASCII to bytes"
			print "/ASCIIHexDecode filter"
		}
		else
		{
			print "(" $4 ") (r) file		% open the file"
		}
		print "/DCTDecode filter		% filter the image data"
		print "false					% multi			pull channels from separate sources)"
		print "3						% ncomp			3 color channels (RGB)"
		print "colorimage				% width height bits/comp matrix <datasrc0> multi ncomp - "

		print "%	restore page origin & scale"
		print "grestore"

		# print exped tab at edge of folded page
		exped_tab_x	= avail_x + 2*margin_x

		# set wide page offset
		wide_page	= 2*(page_x-margin_x)

		# detect any printable lines after graphic
		lines		= max_lines
	}

	# save format
	last_format = format
}
