# AWK script to create filofax table inserts
# v1.0 2024/04/30  k.j.beeden

# awk -f table.awk <definition>.fmt <pagesize> <plain|spine> [embed] > <output>.ps

# options:
# 	<pagesize> is mandatory, and can be [a4|a5|a6|a6p|a7|b7|f5|f6|f6a|p4|p6|mi|m7|q7|A4|A5|A6|A6P|A7|B7|F5|F6|3F6|F6A|P4|P6|MI|M7|Q7]
#		lower case is booklet format
#		upper case is notelet
#		Ax		Ax paper
#		A6		A6 paper 6-hole (1.75" gap)
#		A6P		A6 paper Personal 6-hole (2" gap)
#		B7		Daiso Pocket B7 6-hole
#		F5	 	Filofax A5 Clipbook 6-hole
#		F6	 	Filofax Personal 6-hole (non-Ax page size)
#		3F6	 	Filofax Personal 6-hole triple page (non-Ax page size)
#		F6A	 	Filofax PersonalA 6-hole (A6 page width)
#		P4		Filofax PocketA 4-hole
#		P6		Filofax PocketA 6-hole
#		MI		Filofax Mini (narrower A7) 5-hole
#		M7		Filofax MiniA (A7) 5-hole
#		Q7		QL binder (wider A7) 3-hole
#	<plain|spine> is mandatory
#		plain	no spine margin text
#		spine	spine margin text
#	[embed] is optional
#		embed	embed JPEG images for direct PostScript printing



BEGIN {
	# #####################################
	# Setup
	# script version
	script_file		= "table.awk"
	script_version	= "v1.0"
	script_path		= "../scripts/" script_file

	# text colour
	body_colour		= "text"		# set colour of body text

	# get file dates
	get_dates()

	# decode options
	decode_options()

	# set page properties
	set_page_properties()

	# working variables
	format		= ""
	last_format	= ""
	text		= ""

	area		= 0		# table area counter
	cols		= 0
	rows		= 0

	# print PS file header and prolog
	print_ps_header()

	# set field separator to tab
	FS	= "\t"

}	# END setup


# #####################################
# Parse formatted text lines
{
	# get command and text string
	format	= $1
	text	= substr($0, index($0, ">")+2)

#	###################
	# parse embedded document options
	parse_doc_options(format, text)

	# process new document page
	new_page(format)

	# implement table features
	table_features(format, text)
}


# #####################################
#
END {
	# terminate last page
	print "% undo offset to bottom left of table"
	# undo table translate
	if ((page % 2) == 1)
		print "binding_margin neg footer_margin neg translate"
	else
		print "page_margin neg footer_margin neg translate"

	print ""
	print "showpage"
	print ""

	# document trailer
	print "%%Trailer"
	print "%%Pages:", page
}


# #####################################
#
function get_dates()
{
	dates = "JanFebMarAprMayJunJulAugSepOctNovDec"
	# get today's date in known format
	"date +'%a %b %d %T %m %Y'" | getline
	month		= $5
	day			= $3
	thisyear	= $6
	# date string for PostScript creation date (yyyy/mm/dd)
	today		= sprintf("%04d/%02d/%02d", thisyear, month, day)
	# date string for printing record (hh:mm:ss DOW DOM Mon yyyy)
	time		= sprintf("%s %s %s %s %s", $4, $1, $3, $2, $6)

	# get text filedate (can't use FILENAME variable, as doesn't exist in BEGIN section)
	command		= "ls --fu " ARGV[1]
	command | getline
	# check for current year
	year		= substr($6, 1, 4)
	month		= substr($6, 6, 2)
	day			= substr($6, 9, 2)
	filedate	= sprintf("%04d/%02d/%02d", year, month, day)

#	# get script file date
#	command 	= "ls --fu " script_path
#	command | getline
#	year		= substr($6, 1, 4)
#	month		= substr($6, 6, 2)
#	day			= substr($6, 9, 2)
#	scriptdate	= sprintf("%04d/%02d/%02d", year, month, day)
}

function decode_options()
{
	if (ARGC > 2)
	{
		# decode A size paper
		if (("a4a5a6a7A4A5A6A7" ~ ARGV[2]) && (length(ARGV[2])==2))
		{
			paper = sprintf("A%d", substr(ARGV[2],2))
		}
		# decode A6P paper
		if ((ARGV[2] == "a6p") || (ARGV[2] == "A6P"))
			paper = "A6P"
		# decode B7 paper
		if ((ARGV[2] == "b7") || (ARGV[2] == "B7"))
			paper = "B7"
		# decode QL binding paper
		if ((ARGV[2] == "q7") || (ARGV[2] == "Q7"))
			paper = "Q7"
		# decode Filofax A5 six hole
		if ((ARGV[2] == "f5") || (ARGV[2] == "F5"))
			paper = "F5"
		# decode Filofax Personal six hole
		if ((ARGV[2] == "f6") || (ARGV[2] == "F6"))
			paper = "F6"
		# decode Filofax Triple Personal six hole
		if ((ARGV[2] == "3f6") || (ARGV[2] == "3F6"))
			paper = "3F6"
		# decode Filofax PersonalA six hole
		if ((ARGV[2] == "f6a") || (ARGV[2] == "F6A"))
			paper = "F6A"
		# decode Filofax Pocket four hole
		if ((ARGV[2] == "p4") || (ARGV[2] == "P4"))
			paper = "P4"
		# decode Filofax Pocket six hole
		if ((ARGV[2] == "p6") || (ARGV[2] == "P6"))
			paper = "P6"
		# decode Filofax MiniA 5-hole
		if ((ARGV[2] == "m7") || (ARGV[2] == "M7"))
			paper = "M7"
		# decode Filofax Mini 5-hole
		if ((ARGV[2] == "mi") || (ARGV[2] == "MI"))
			paper = "MI"

		# check for booklet cover
		if ("a4a5a6a6pb7f6p4pm7mi6q7" ~ ARGV[2])
		{
			book = 1
		}
		# check for spine option
		if (ARGC > 3)
			if (ARGV[3] == "spine")
				# enable spine
				spine	= 1

		# check to see if JPEG embed option set
		if (ARGC > 4)
		{
			if (ARGV[4] == "embed")
				# enable JPEG embedding
				embed	= 1
		}

		# leave only script name and filename in arguments array
		ARGC	= 2
	}
}

function set_page_properties()
{
	# default values
	page_x		= 210				# printed width
	page_y		= 297				# printed height
	max_lines	= 23
	Q7_bind		= 0
	ring_radius	= 2					# binding

	switch (paper)
	{
		case "A4":
			paper_x		= 210				# actual width
			paper_y		= 297				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 80				# centre spacing
			ring_space	= 80				# ring spacing
			ring_group	= 0					# #rings in each group
			ring_radius	= 3					# binding
			break							# end case
		case "A5":
			paper_x		= 148.5				# actual width
			paper_y		= 210				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 69.85				# centre spacing 2.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			ring_radius	= 2.5				# binding
			break							# end case
		case "A6":							# A6 6-hole 1.75" gap
			paper_x		= 105				# actual width
			paper_y		= 148.5				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 38.10				# centre spacing 1.5"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "A6P":							# A6 Filofax Personal punching 6-hole (2" gap)
			paper_x		= 105				# actual width
			paper_y		= 148.5				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "A7":							# A7 stitching
			paper_x		= 74.25				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 3.54				# ring offset 
			ring_gap	= 0					# centre spacing 0 
			ring_space	= 3.89				# ring spacing 
			ring_group	= 8					# #rings in each group
			ring_radius	= 0.177				# sewn binding
			break							# end case
		case "B7":
			paper_x		= 88.29				# actual width (A7 * 4rt(2))
			paper_y		= 124.87			# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 19.05				# centre spacing 0.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "F5":							# Filofax Clipbook A5
			paper_x		= 148.5				# actual width
			paper_y		= 210				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 69.85				# centre spacing 2.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "F6":							# Filofax Personal
			page_x		= 210				# printed width		(Filofax Personal width 3.75", 95mm)
			page_y		= 378				# printed height	(Filofax Personal height 6.75", 171mm)
			paper_x		= 95				# actual width
			paper_y		= 171				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			max_lines	= 27
			break							# end case
		case "3F6":							# Filofax Triple page Personal
			page_x		= 590				# printed width		(Filofax Personal width 3.75", 95mm)
			page_y		= 378				# printed height	(Filofax Personal height 6.75", 171mm)
			paper_x		= 267				# actual width (95 + 2 * (95-9))
			paper_y		= 171				# actual height 
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			max_lines	= 27
			break							# end case
		case "F6A":							# Filofax Personal
			page_x		= 210				# printed width		(A6 width 105mm)
			page_y		= 342				# printed height	(Filofax Personal height 6.75", 171mm)
			paper_x		= 105				# actual width
			paper_y		= 171				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			max_lines	= 27
			break							# end case
		case "P4":							# Filofax PocketA 4-hole
			paper_x		= 85.31				# actual width
			paper_y		= 120.65			# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 2					# #rings in each group
			break							# end case
		case "P6":							# Filofax PocketA 6-hole
			paper_x		= 85.31				# actual width
			paper_y		= 120.65			# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 19.05				# centre spacing 0.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "Q7":							# QL wide A7 3-hole
			page_x		= 226				# printed width
			page_y		= 297				# printed height
			paper_x		= 79.90				# actual width
			paper_y		= 105				# actual height
			Q7_bind		= 16				# binding margin for use with QL ring binder
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 0					# centre spacing 0
			ring_space	= 38.10				# ring spacing 1.5"
			ring_group	= 2					# #rings in each group
			break							# end case
		case "M7":							# Filofax MiniA A7 5-hole
			paper_x		= 74.25				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 0					# centre spacing 0
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "MI":							# Filofax Mini narrow A7 5-hole
			page_x		= 190				# printed width
			page_y		= 297				# printed height
			paper_x		= 67.50				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 0					# centre spacing 0
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		default:
			paper_x		= 210				# actual width
			paper_y		= 297				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 80				# centre spacing
			ring_space	= 80				# ring spacing
			ring_group	= 0					# #rings in each group
			ring_radius	= 3					# binding
			break							# end case
    }

	# calculate paper scale
	paper_scale	= paper_y / page_y	# printed to actual scale
}

function print_ps_header()
{
	print "%!PS-Adobe-3.0"
	print "%%Title:		"paper, "Filofax Table"
	print "%%Creator:		table.awk", script_version, "copyright 2024-" thisyear, "paranoiaproducts"
	print "%%CreationDate:	" today
	if (paper == "Q7")
		printf("%%%%BoundingBox:	0 0 %d %d\n", (page_x + Q7_bind) * 72/25.4, page_y * 72/25.4)
	else
		printf("%%%%BoundingBox:	0 0 %d %d\n", page_x * 72/25.4, page_y * 72/25.4)
	print "%%Pages: (atend)"
	print "%%LanguageLevel:	3"
	print "%%EndComments"
	print "%%BeginProlog"
	
	# dump parameters
	print "% imposition parameters"
	print "/paper\t\t\t(" paper ") def"
	print "/a4_x\t\t\t" 210, "def"
	print "/a4_y\t\t\t" 297, "def"
	print "/page_x\t\t\t" page_x, "def"
	print "/page_y\t\t\t" page_y, "def"
	print "/paper_x\t\t" paper_x, "def"
	print "/paper_y\t\t" paper_y, "def"
	print "/paper_scale\t" paper_scale, "def"
	print "/Q7_bind\t\t" Q7_bind, "def"
	print "/ring_margin\t" ring_margin, "def"
	print "/ring_gap\t\t" ring_gap, "def"
	print "/ring_space\t\t" ring_space, "def"
	print "/ring_group\t\t" ring_group, "def"
	print "/ring_radius\t" ring_radius, "def"
	print ""

	# include fixed functions

	system("cat ../scripts/postscript/tables_hdr.eps")

	print "% end of inserts"
	print ""
	print ""
	print "% specific page definition values"
}

function parse_doc_options(format, text)
{
	# decode page setup options
	# margins
	# all margins, absolute, in mm
	if (format == "<mg>")
	{
		print "% margins"
		binding_margin	= $2 / paper_scale
		page_margin		= $3 / paper_scale
		footer_margin	= $4 / paper_scale
		top_margin		= $5 / paper_scale
		# derived values, scaled
		extent_x		= page_x - binding_margin - page_margin
		extent_y		= page_y - footer_margin - top_margin
		print "% page margins"
		print "/binding_margin	" binding_margin, "def"
		print "/page_margin	" page_margin, "def"
		print "/footer_margin	" footer_margin, "def"
		print "/top_margin		" top_margin, "def"
		print "% page extent"
		print "/extent_x		" extent_x, "def"
		print "/extent_y		" extent_y, "def"
	}

	# page orientation (portrait or landscape)
	if (format == "<or>")
	{
		# report margins
		print "% binding_margin\t" binding_margin
		print "% page_margin\t\t" page_margin
		print "% top_margin\t\t" top_margin
		print "% footer_margin\t" footer_margin

		# check to see if orientation already set within this page
		if (orientation != "")
		{
			# check to see if this a re-orientation from landscape within a page
			if ((orientation == "landscape") && ($2 == "portrait"))
			{
				print "% undo landscape orientation"
				if ((page % 2) == 1)
				{
					# odd page
					if (orientation == "landscape")
						print 0, extent_y, "translate 0 0 moveto -90 rotate"
				}
				else
				{
					# even page
					if (orientation == "landscape")
						print extent_x, 0, "translate 0 0 moveto 90 rotate"
				}
			}
			orientation		= $2
			print "% orientation", orientation
			# derive table extent
			if ($2 == "portrait")
			{
				extent_x		= page_x - binding_margin - page_margin
				extent_y		= page_y - footer_margin - top_margin
			}
			else
			{
				extent_y		= page_x - binding_margin - page_margin
				extent_x		= page_y - footer_margin - top_margin
			}
		}
		else
		{
			orientation		= $2
			print "% orientation", orientation
			# derive table extent
			if ($2 == "portrait")
			{
				extent_x		= page_x - binding_margin - page_margin
				extent_y		= page_y - footer_margin - top_margin
			}
			else
			{
				extent_y		= page_x - binding_margin - page_margin
				extent_x		= page_y - footer_margin - top_margin
			}
			print "%	extents", extent_x, extent_y
			if (orientation == "landscape")
			{
				print "% change to landscape"
				if ((page % 2) == 1)
				{
					# odd page
					print "0 0 moveto 90 rotate", 0, -extent_y, "translate"
				}
				else
				{
					# even page
					print "0 0 moveto -90 rotate", -extent_x, 0, "translate"
				}
			}
			print ""
		}
	}

	# spine text
	if (format == "<st>")
	{
		print "% spine text"
		print "/spine_text", text, "def"
	}
}

function new_page(format)
{
	# new page for unnumbered & numbered pages
	if (format == "<p0>")
	{
		# check for first page (end of prolog)
		if (page == 0)
			close_prolog()

		#	finish previous page
		# undo table offset
		if (page > 0)
		{
			print "%	undo offset to bottom left of table"
			if ((page % 2) == 1)
			{
				# odd page
				if (orientation == "landscape")
					print 0, extent_y, "translate 0 0 moveto -90 rotate"
				print "binding_margin neg"
			}
			else
			{
				# even page
				if (orientation == "landscape")
					print -extent_x, 0, "translate 0 0 moveto 90 rotate"
				print "page_margin neg"
			}
			print "footer_margin neg"
			print "translate"
			print ""
			print "showpage"
			print ""
		}


		# Start new page
		# increment physical and numbered page counts
		page++

		# reset area & cols counters
		area		= 0
		cols		= 0
		orientation	= ""

		# note the page type
		page_format	= format

		# print sheet header
		print ""
		print "%%Page:", page, page
		print "%%BeginPageSetup"
		print "% convert to mm scaling"
		print "72 25.4 div dup scale"
		# add printer offset correction on even sheets
		if ((sheet % 2) == 1)
		{
			print "% printer registration correction"
			print "oddCorrect"
		}
		else
		{
			print "% printer registration correction"
			print "evenCorrect"
		}

		print "%%EndPageSetup"

		# setup page 
		{
			# document page
			print "% document page", page

			# colour, spine & binding
			print "% colour, spine & binding"
			print body_colour

			# print spine for odd or even pages
			if (spine == 1)
				if ((page % 2) == 1)
					print "lspine"	# odd page
				else
					print "rspine"	# even page

			# if enabled, print binding holes every 8 sheets
			if ((page % 8) == 1)
			{
				print "(" paper ") binding"
			}

			# offset to table
			print "% offset to bottom left of table"
			if ((page % 2) == 1)
			{
				# odd page
				print "binding_margin"
			}
			else
			{
				# even page
				print "page_margin"
			}
			print "footer_margin"
			print "translate"
			print ""

			print "% start decoding drawn element commands"
			print ""
		}
	}		# end new page
}

function close_prolog()
{
	# write derived values
	print ""
	print "% derived values"

	print "%%EndProlog"
	print ""

	# select PostScript document tumble printing mode
	print "%%BeginSetup"
	print "<< /Duplex true /Tumble false >> setpagedevice"
	print ""

	print "%%EndSetup"
}

function table_features(format, text)
{
	# page outline
	if (format == "<po>")
	{
		# draw page outline
		print "% draw page outline"
		print "gsave"
		# check for line thickness parameter
		if (NF == 2)
			print $2 / paper_scale, "setlinewidth"
		# set line end and join styles
		print "2 setlinecap 0 setlinejoin [] 0 setdash"
		print "0 0 moveto"
		print extent_x, 0, "lineto"
		print extent_x, extent_y, "lineto"
		print 0, extent_y, "lineto"
		print "closepath stroke"
		print "grestore"
		print ""
	}

	# table area definition, as % of table extent
	#	expects definition file to be sensibly hierarchical
	if (format == "<ad>")
	{
		area++
		print "% area definition", area
		values[area, "area_x"]			= $2 * extent_x / 100
		values[area, "area_y"]			= $3 * extent_y / 100
		print "%	area", area
		print "%	area size", values[area, "area_x"], values[area, "area_y"]
		values[area, "pos_x"]			= $4 * extent_x / 100
		values[area, "pos_y"]			= $5 * extent_y / 100
		print "%	area position", values[area, "pos_x"], values[area, "pos_y"]
		# initialise column definitions
		cols	= 0
		values[area, 0, "col_pos_x"]	= 0		# left
		values[area, 1, "col_pos_x"]	= values[area, "area_x"]	# right
		print ""
	}

	# table area outline
	if (format == "<ao>")
	{
		# draw area outline
		print "% draw area outline", area
		print "gsave"
		# check for line thickness parameter
		if (NF == 2)
			print $2 / paper_scale, "setlinewidth"
		# set line end and join styles
		print "2 setlinecap 0 setlinejoin [] 0 setdash"
		print values[area, "pos_x"], values[area, "pos_y"], "moveto"
		print values[area, "area_x"], 0, "rlineto"
		print 0, values[area, "area_y"], "rlineto"
		print -values[area, "area_x"], 0, "rlineto"
		print "closepath stroke"
		print "grestore"
		print ""
	}

	# define number of rows in this area
	if (format == "<rd>")
	{
		print "% row definition", $2
		values[area, "rows"]	= $2
		# derived values
		values[area, "row_y"]	= values[area, "area_y"] / $2
		print "%	rows", values[area, "rows"]
		print "%	row_y", values[area, "row_y"]
		print ""
	}

	# draw row lines
	#	<rl> min_r max_r min_col max_col
	if (format == "<rl>")
	{
		print "% draw row lines"
		# check not exceeding row & col counts (allow 1 over for drawing final row)
		if (($2-1 <= values[area, "rows"]) && ($3-1 <= values[area, "rows"]) && ($4-1 <= values[area, "cols"]) && ($5-1 <= values[area, "cols"]))
		{
			# get values
			#	x start position
			# left side of first column (right side of previous column)
			start_x	= values[area, "pos_x"] + values[area, $4-1, "col_pos_x"]

			#	x stop position
			# right side of last column
			stop_x	= values[area, "pos_x"] + values[area, $5, "col_pos_x"]

			#	y start position
			start_y	= values[area, "pos_y"] + values[area, "area_y"] - ($2-1) * values[area, "row_y"]

			# for each row
			for (row = $2-1; row < $3; row++)
			{
				print "% row", row
				print start_x, start_y, "moveto"
				print stop_x, start_y, "lineto"
				start_y	= start_y - values[area, "row_y"]
			}
			print "stroke"
		}
		else
			print FILENAME, "row line: too many rows", $0 | "cat 1>&2"
		print ""
	}

	# column definitions (assumes all columns defined L-R, in order)
	#	horizontal position % of row_width
	if (format == "<cd>")
	{
		print "% column definitions"
		# record each column
		for (field = 2; field <= NF; field++)
		{
			the_col		= field - 1
			# record number of columns & this column horizontal right margin
			values[area, the_col, "col_pos_x"]	= values[area, "area_x"] * $field / 100
			print "%	col_pos_x", the_col, values[area, the_col, "col_pos_x"]
		}
		# ensure last column is 100%, and record number of columns
		if ($NF != 100)
		{
			values[area, NF, "col_pos_x"]	= values[area, "area_x"]
			print "%	col_pos_x", NF, values[area, NF, "col_pos_x"]
			values[area, "cols"]			= NF
		}
		else
			values[area, "cols"]			= NF-1
		print ""
	}

	# column lines (assumes all columns defined by <cd>)
	#	horizontal position % of col, start_row, end_row [start_row_n, end_row_n]
	#	rows defined top down
	if (format == "<cl>")
	{
		the_col		= $2
		# check column defined
		if (the_col <= values[area, "cols"])
		{
			print "% draw column", the_col, "lines"
			# get start & stop pairs
			for (param = 3; param < NF; param=param+2)
			{
				# check not exceeding row counts
				if (($param <= values[area, "rows"]) && ($(param+1) <= values[area, "rows"]))
				{
					# draw lines
					#	col x, top of start row in area 
					print values[area, "pos_x"] + values[area, the_col, "col_pos_x"], values[area, "pos_y"] + values[area, "area_y"] - ($param-1) * (values[area, "row_y"]), "moveto"
					#	vertical line, length of number of rows
					print 0, ($(param) - $(param+1) - 1) * values[area, "row_y"], "rlineto stroke"
				}
				else
					print "column: too many rows", $0 | "cat 1>&2"
			}
			print ""
		}
		else
			print "column not defined:", $0 | "cat 1>&2"
	}

	# border thickness (absolute (mm))
	if (format == "<bt>")
	{
		print "% border line thickness"
		print $2/paper_scale, "setlinewidth"
		print ""
	}

	# border format (colour)
	if (format == "<bf>")
	{
		print "% border line colour"
		print $2
	}

	# border line style (cap, join, dash)
	if (format == "<bs>")
	{
		print "% border line style"
		print $2
		print ""
	}
	
	# cell colour fill
	#	<fi> row col <colourname|postscript colour definition>
	#	<fi> min_col max_col min_row max_row <colourname|postscript colour definition>
	if (format == "<fi>")
	{
		print "% colour fill"
		print "gsave"
		# multiple cells?
		if (NF == 6)
		{
			# check bounds valid
			if (($3 > values[area, "cols"]) || ($5 > values[area, "rows"]))
				print "fill area bounds invalid:", values[area, "rows"], values[area, "cols"], $0 | "cat 1>&2"
			else
			{
				print "% multi-cell fill"
				start_x		= values[area, "pos_x"] + values[area, $3, "col_pos_x"]
				start_y		= values[area, "pos_y"] + values[area, "area_y"] - $5 * values[area, "row_y"]
				delta_x		= values[area, $2-1, "col_pos_x"] - values[area, $3, "col_pos_x"]
				delta_y		= values[area, "row_y"] * ($5-$4+1)
				colour		= $6
			}
		}
		else
		{
			print "% single cell fill"
			start_x		= values[area, "pos_x"] + values[area, $2, "col_pos_x"]
			start_y		= values[area, "pos_y"] + values[area, "area_y"] - $3 * values[area, "row_y"]
			delta_x		= values[area, $3-1, "col_pos_x"] - values[area, $2, "col_pos_x"]
			delta_y		= values[area, "row_y"]
			colour		= $4
		}
		# draw box
		print start_x, start_y, "moveto"
		print delta_x, 0, "rlineto"
		print 0, delta_y, "rlineto"
		print -delta_x, 0, "rlineto"
		# close path, set colour & fill
		print "closepath"
		print colour
		print "fill"
		print "grestore"
		print ""
	}

	# text function
	# font definitions
	#	font face
	if (format == "<fn>")
	{
		print "% font name"
		values[area, "font"]	= $2
	}
	# absolute font size & offsets (mm)
	if (format == "<fa>")
	{
		print "% font size % offsets (absolute)"
		values[area, "font_size"]	= $2
		values[area, "font_xoff"]	= $3
		values[area, "font_yoff"]	= $4
	}
	# relative font size & offsets (% of row height)
	if (format == "<fr>")
	{
		print "% font size % offsets (relative)"
		# derived values
		values[area, "font_size"]	= $2 * values[area, "row_y"] / 100
		values[area, "font_xoff"]	= $3 * values[area, "row_y"] / 100
		# text vertically centred in row
		values[area, "font_yoff"]	= (values[area, "row_y"] * (1 - $2/140)) / 2
	}
	# select the font as defined above
	if (format == "<fs>")
	{
		print "% activate font"
		print values[area, "font"], values[area, "font_size"], "selectfont"
		print ""
	}

	# print text string in selected row, column position
	#	<tx>	[<](string)[>] row col1 [col2]
	#	<(string)	left justified
	#	<(string)>	centred
	#	(string)>	right justified
	#	if there are two col parameters, will position between the two
	#		use for centre justified
	if (format == "<tx>")
	{
		# top of area - rows + text offset
		text_y	= (values[area, "pos_y"] + values[area, "area_y"]) - ($3 * values[area, "row_y"]) + values[area, "font_yoff"]

		# horizontal position
		left_x	= values[area, "pos_x"] + values[area, $4-1, "col_pos_x"]
		if (NF == 5)
			right_x	= values[area, "pos_x"] + values[area, $5, "col_pos_x"]
		else
			right_x	= values[area, "pos_x"] + values[area, $4, "col_pos_x"]

		print "% string"

		# left justified
		if ((substr($2, 1, 1) == "<") && (substr($2, length($2), 1) != ">"))
		{
			text_x	= left_x + values[area, "font_xoff"]
			print text_x, text_y, "moveto"
			print substr($2, 2), "show"
		}

		# centre justified
		if ((substr($2, 1, 1) == "<") && (substr($2, length($2), 1) == ">") )
		{
			text_x	= (left_x + right_x) / 2
			print text_x, text_y, "moveto"
			print substr($2, 2, length($2)-2), "centreshow"
		}

		# right justified
		if ((substr($2, 1, 1) != "<") && (substr($2, length($2), 1) == ">")) 
		{
			text_x	= right_x - values[area, "font_xoff"]
			print text_x, text_y, "moveto"
			print substr($2, 1, length($2)-1), "rightshow"
		}

		# title
		if (substr($2, 1, 1) == ">") 
		{
			if ((page % 2) == 1)
			{
				# odd page
				print "% odd page title flush right"
#				print "% extent_x extent_y", extent_x, extent_y
				text_x	= extent_x - values[area, "font_xoff"]
				text_y	= extent_y + values[area, "font_yoff"]
				print text_x, text_y, "moveto"
				print substr($2, 2, length($2)-1), "rightshow"
			}
			else
			{
				# even page
				print "% even page title flush left"
				print "% extent_x extent_y", extent_x, extent_y
				text_x	= values[area, "font_xoff"]
				text_y	= extent_y + values[area, "font_yoff"]
				print text_x, text_y, "moveto"
				print substr($2, 2, length($2)-1), "show"
			}
		}
		print ""
	}

	# Area fill functions
	#
	# fill area with bujo
	#	row_y spacing in x and y
	#	row_y starting position for x & y
	#	max_y top of area, less half row_y, to prevent drawing dot grid on margin
	if (format == "<bj>")
	{
		if (NF == 1)
		{
			# fill entire area with bujo grid
			#	min_y max_y min_x max_x step
			print "% fill area with bujo grid, at row_y spacing"
			print values[area, "pos_y"] + values[area, "row_y"], values[area, "pos_y"] + values[area, "area_y"]-values[area, "row_y"]/2, values[area, "pos_x"] + values[area, "row_y"], values[area, "pos_x"] + values[area, "area_x"], values[area, "row_y"], "bujo"
			print ""
		}
		else
		{
			# fill specific area with bujo
			print "% fill specific area with bujo grid, at row_y spacing"
print "bujo: oops - sub-area fill not yet implemented, line", NR, $0 | "cat 1>&2"
			print ""
		}
	}

	# graphic: EPS
	# ge size_x size_y filename
	# should be last element in a page
	if (format == "<ge>")
	{
		print "% insert EPS image:", $4
		# get image dimensions
		pic_x	= $2
		pic_y	= $3
		print "%	dimensions:", pic_x, pic_y

		# available space
		avail_x	= values[area, "area_x"]
		avail_y	= values[area, "area_y"]

		# fit image into available space
		#	between page left/right margins
		scale_x	= avail_x / pic_x
		#	between bottom of text & bottom of page
		scale_y	= avail_y / pic_y

		# choose smaller scale & place graphic
		if (scale_x < scale_y)
		{
			# fill in x dimension
			clip_x			= scale_x * pic_x
			clip_y			= scale_x * pic_y
			# no gap in x dimension
			pic_x_offset	= 0
			# add gap in y dimension
			pic_y_offset	= (avail_y - clip_y)/2
			the_scale		= scale_x
		}
		else
		{
			# fill in y dimension
			clip_x			= scale_y * pic_x
			clip_y			= scale_y * pic_y
			# add gap in x dimension
			pic_x_offset	= (avail_x - clip_x)/2
			# no gap in y dimension
			pic_y_offset	= 0
			the_scale		= scale_y
		}

		# set scale and origin
		print "%	available image space:", avail_x, avail_y
		print "%	scaled image space:", clip_x, clip_y
		print "%	save current scale & origin"
		print "gsave"
		print "%	translate to bottom left corner of image"
		print values[area, "pos_x"] + pic_x_offset, values[area, "pos_y"] + pic_y_offset, "translate"
		print "%	scale to fit larger dimension in available space"
		print the_scale, the_scale, "scale"

		# blank background
		print "%	blank image background"
		print "white"
		print "newpath"
		print "0 0 moveto"
		print pic_x, 0, "rlineto"
		print 0, pic_y, "rlineto"
		print -pic_x, 0, "rlineto"
		print "closepath"
		print "fill"
		print "black"

		# insert image
		print "%	insert image"
		printf("(%s) run\n", $4)

		print "%	restore page origin & scale"
		print "grestore"
		print ""

		# detect any printable lines after graphic
		lines	= max_lines
	}

	# graphic: JPEG
	# gj size_x size_y filename
	if (format == "<gj>")
	{
		print "% insert JPEG image:", $4
		# get image dimensions
		pic_x	= $2
		pic_y	= $3
		print "%	dimensions:", pic_x, pic_y
		# available space
		avail_x	= values[area, "area_x"]
		avail_y	= values[area, "area_y"]
		print "%	available image space:", avail_x, avail_y
		# fit image into available space
		# between page left/right margins
		scale_x	= avail_x / pic_x
		# between bottom of text & bottom of page
		scale_y	= avail_y / pic_y

		# choose smaller scale & calculate scaled image dimensions
		if (scale_x < scale_y)
		{
			# fill in x dimension
			clip_x			= scale_x * pic_x
			clip_y			= scale_x * pic_y
			# no gap in x dimension
			pic_x_offset	= 0
			# add gap in y dimension
			pic_y_offset	= (avail_y - clip_y)/2
			the_scale		= scale_x
		}
		else
		{
			# fill in y dimension
			clip_x			= scale_y * pic_x
			clip_y			= scale_y * pic_y
			# add gap in x dimension
			pic_x_offset	= (avail_x - clip_x)/2
			# no gap in y dimension
			pic_y_offset	= 0
			the_scale		= scale_y
		}

		print "%	scaled image space:", clip_x, clip_y
		print "%	save current scale & origin"
		print "gsave"
		print "%	translate to bottom left corner of image"
		print values[area, "pos_x"] + pic_x_offset, values[area, "pos_y"] + pic_y_offset, "translate"
		print "%	scale to fit larger dimension in available space"
		print the_scale, the_scale, "scale"

		# scale to unit image
		print "%	scale to make unit image fit"
		print pic_x, pic_y, "scale"

		# print image
		print "%	read image into unit square"
		print "%	parameters for colorimage"
		print pic_x, "					% width"
		print pic_y, "					% height"
		print 8, "						% 8 bits per color channel"
		print "%	[width 0 0 -height 0 height] scan L-R, B-T"
		print "[" pic_x, 0, 0, -pic_y, 0, pic_y "] % matrix		transform array... maps unit square to pixel"
		if (embed == 1)
		{
			print "%	embed JPEG as string"
			print "("
			system("hexdump -v -e '64/1 \"%02x\" \"\n\"' " $4)
#			system("hexdump -v -e '/1 \"%02x\"' " $4)
			print ")"
			print "%	convert ASCII to bytes"
			print "/ASCIIHexDecode filter"
		}
		else
		{
			print "(" $4 ") (r) file		% open the file"
		}
		print "%	decode DCT to image"
		print "/DCTDecode filter		% filter the image data"
		print "false					% multi			pull channels from separate sources)"
		print "3						% ncomp			3 color channels (RGB)"
		print "colorimage				% width height bits/comp matrix <datasrc0> multi ncomp - "

		print "%	restore page origin & scale"
		print "grestore"
		print ""
	}
}

