# AWK script to create filofax diary from 'cal' output 
# v1.0 2024/03/29 k.j.beeden
# awk -f wo1p.awk <year.txt> <pagesize> <spine> <hor_sub> <vert_sub> > output.ps
#
# <pagesize>	[A4|A5|A6|A6P|A7|B7|F5|F6|F6A|P4|P6|Q7|M7|MI]
# <spine>		[spine|blank] 	# print spine details
# <hor_sub>		[0|value]		# choose number of horizontal sub-divisions (0=none)
# <vert_sub>	[0|value]		# choose number of vertical sub-divisions (0=none)
#
# cal -Y -m -w --iso 1 <year>
# <format>
#       January 2024             February 2024               March 2024      
#   Mo Tu We Th Fr Sa Su      Mo Tu We Th Fr Sa Su      Mo Tu We Th Fr Sa Su
# 1  1  2  3  4  5  6  7    5           1  2  3  4    9              1  2  3
# 2  8  9 10 11 12 13 14    6  5  6  7  8  9 10 11   10  4  5  6  7  8  9 10
# 3 15 16 17 18 19 20 21    7 12 13 14 15 16 17 18   11 11 12 13 14 15 16 17
# 4 22 23 24 25 26 27 28    8 19 20 21 22 23 24 25   12 18 19 20 21 22 23 24
# 5 29 30 31                9 26 27 28 29            13 25 26 27 28 29 30 31
#                                                                   
# <end format>
#|                         |                         |
#0        1         2         3         4         5         6         7     
#123456789012345678901234567890123456789012345678901234567890123456789012345

# known problems
# ##############
# if any weeks from preceding year are present (i.e. first week isn't 1), the split month logic fails
#	edit cal output to blank that week
# if end of year falls into week 1 of following year, fails to print
#	edit cal output to make last week 53
# can only cope with weeks 1-53
#	does fill rest of week 53/1 if there is no data beyond end of year
#
# ###############
# unix cal command prints a month calendar
# cal -Y -m -w --iso 1 2024
# shows entire upcoming year from January 2024
#
# cal -n 15 -m -w --iso 12 2023
# shows 15 months, starting at December 2023
# this could be used to overcome the problem of unknown dates from previous & following years
# would need change to processing to cope with weeks from multiple years
#
# calendar
# BSD package can be installed in debian and ubuntu from bsdmainutils
# appears to be a BSD utility to print holidays
#
# calendarific appears to be commercial service providing holiday dates
# has a free API for single user
# https://calendarific.com/pricing
# 
# MS Azure appears to have a dataset, covering up to 2099
# https://learn.microsoft.com/en-us/azure/open-datasets/dataset-public-holidays?tabs=azureml-opendatasets

BEGIN {
	# #####################################
	# Setup
	# script version
	script_file		= "wo1p.awk"
	script_version	= "v1.0"
	script_path		= "../scripts/" script_file

	# decode options
	decode_options()

	# set page size
	set_page_properties()

	# get file dates
	get_dates()
	the_year	= substr(filename, 4, index(filename, ".t")-4)


	# ############################################
	# define values for layout
	#
	# printing margins (values in absolute mm)
	print_margin 	= 3		# 'print-to-edge' limit
	binding_margin	= 8		# punching margin

	# diary pages properties (values in 'A4 mm': will retain layout across different page sizes)
	cal_width		= 65	# calendar width
	cal_height		= 26	# calendar height
	cal_gap			= 3		# gap between table & calendar
	cal_t_off		= 0.7	# calendar text offset
	mnthyr_offset	= 5		# offset of header month/year from top of table
	range_offset	= 16	# offset of header day range from top of table
	event_offset	= 7		# offset of table event entry from top of box
	rows			= 4		# rows per page

	# multiplicative factors
	t_mgn_factor	= 0.07	# table entry margin width fraction
	hdr_factor		= 1.25	# header to horiz divider factor

	# imposition properties
	tick			= 20	# length of crop mark tick


	# working variables
	max_week	= 0

}	# END setup


# #####################################
# process 'cal' output into week entries
# 	data is in blocks of eight lines per three months

# find month & year from first line in each eight
{
	if ((NR % 8) == 1)
	{
		line_mnth[1]	= $1
		line_year[1]	= $2
		line_mnth[2]	= $3
		line_year[2]	= $4
		line_mnth[3]	= $5
		line_year[3]	= $6
	}
}

# find day name columns from second line in each eight
# (nb. the same for each month)
{
	if ((NR % 8) == 2)
	{
		# column 1 day names
		days[1]		= $1
		days[2]		= $2
		days[3]		= $3
		days[4]		= $4
		days[5]		= $5
		days[6]		= $6
		days[7]		= $7
	}
}

# extract week & dates from non-blank entries in lines 3-8
{
	if (((NR-1) % 8) > 1)
	# lines between 3 and 8
	{
		for (the_mnth = 1; the_mnth < 4; the_mnth++)
		{
			# week number is first two characters of each 26 character block
			# convert string to decimal
			week	=  substr($0, 26*(the_mnth-1)+1, 2) + 0 # cast to number

			# record maximum week
			if (week > max_week)
				max_week	= week

			# record year & month for each valid week
			if (week != 0)
			{
				# record year
				data[week, "wk_year"]	= line_year[the_mnth]

				# record month for each week, merge if already existing
				if (data[week, "wk_mnth"] == "")
					data[week, "wk_mnth"]	= line_mnth[the_mnth]
				else
					if (week == max_week)
						data[week, "wk_mnth"]	= sprintf("%s", substr(data[week, "wk_mnth"], 1 , 3) "/" substr(line_mnth[the_mnth], 1, 3))
					else
						data[week, "wk_mnth"]	= sprintf("%s", substr(line_mnth[the_mnth], 1, 3) "/" substr(data[week, "wk_mnth"], 1 , 3))
				# get dates
				for (day = 1; day < 8; day++)
				{
					# get dates
					date	= substr($0, 26*(the_mnth-1) + 3*day + 1, 2) + 0	# cast to number
					if (date != 0)
						data[week, day]	= date
				}
			}
		}
	}
}


# ############################################
#
# process captured week lines into diary pages

END {
	# clean up week 1
	fill_date	= 31
	for (day = 7; day > 0; day--)
	{
		if (data[1, day] == 0)
		{
			data[1, day] = fill_date
			fill_date--
			data[1, "wk_mnth"]	= sprintf("Dec %4d/Jan", the_year-1)
		}
	}

	# fill weeks 0 to -3
	#	start with day before first day of week 1
	fill_date	= data[1, 1] - 1
	if (fill_date < 1)
		fill_date	+= 31
	fill_mnth	= "December"
	
	for (fill = 0; fill > -4; fill--)
	{
		data[fill, "wk_year"]	= the_year-1
		data[fill, "wk_mnth"]	= fill_mnth
		for (day = 7; day > 0; day--)
		{
			data[fill, day] = fill_date
			fill_date--
			if (fill_date < 1)
			{
				fill_date	+= 30
				if (day == 1)
					fill_mnth	= "November"
				else
					fill_mnth	= "Nov/Dec"
			}
		}
	}

	# clean up week 53
	fill_date	= 1
	data[53, "wk_mnth"]	= "January"
	data[53, "wk_year"]	= the_year+1
	for (day = 1; day < 8; day++)
	{
		if (data[53, day]+0.0 == 0)
		{
			data[53, day] = fill_date
			fill_date++
		}
		else
		{
			data[53, "wk_mnth"]	= "Dec " the_year "/Jan"
			data[53, "wk_year"]	= the_year+1
		}
	}
	
	# fill weeks 54-56
	for (fill = 54; fill < 57; fill++)
	{
		data[fill, "wk_mnth"]	= "January"
		data[fill, "wk_year"]	= the_year+1
		for (day = 1; day < 8; day++)
		{
			data[fill, day] = fill_date
			fill_date++
			if (fill_date > 31)
			{
				fill_date	= 1
				if (day == 7)
					fill_mnth	= "February"
				else
					fill_mnth	= "Jan/Feb"
			}
		}
	}


	# ############################################
	# derived properties (values in 'A4 mm')
	lmargin			= binding_margin / paper_scale			# left margin
	rmargin			= print_margin / paper_scale			# right margin
	header			= cal_height + cal_gap + rmargin		# page header
	footer			= rmargin								# page footer

	# derived properties
	t_width			= page_x - lmargin - rmargin	# table width
	t_height		= page_y - header - footer		# table height
	cal_d_wid		= cal_width / 8.25				# width of calendar date entry
	cal_w_wid		= cal_d_wid * 1.25				# width of calendar week entry
	cal_d_hgt		= cal_height / 6				# height of calendar row
	cal_y			= t_height + cal_gap			# calendar y position
	rcal_x			= t_width - cal_width			# right calendar x position
	box_height		= t_height / rows
	box_width		= t_width / 2
	if (hor_sub == 0)
		hdr_height	= hdr_factor * t_height / ((5+hdr_factor) * rows)
	else
		hdr_height	= hdr_factor * t_height / ((hor_sub+hdr_factor) * rows)

	# fixed text strings
	weekdays[0]		= "(Week)"
	weekdays[1]		= "(Monday)"
	weekdays[2]		= "(Tuesday)"
	weekdays[3]		= "(Wednesday)"
	weekdays[4]		= "(Thursday)"
	weekdays[5]		= "(Friday)"
	weekdays[6]		= "(Saturday)"
	weekdays[7]		= "(Sunday)"
	cal_days		= "WkMoTuWeThFrSaSu"


	# ############################################
	# print PS file header and prolog
	print_ps_header()

	# print results
	print "% decoded cal output"
	for (the_week = -3; the_week < 57; the_week++)
		printf("%% %4d %2d %12s %2d %2d %2d %2d %2d %2d %2d\n", data[the_week, "wk_year"], the_week, data[the_week, "wk_mnth"], data[the_week, 1], data[the_week, 2], data[the_week, 3], data[the_week, 4], data[the_week, 5], data[the_week, 6], data[the_week, 7])
	print "% max_week", max_week

	# force odd number of weeks to ensure even number of printed pages
	max_week	= 53


	# ############################################
	# create pages from extracted calendar entries

	# title page to force Mon-Wed on second page
	title_page()

	# create week diary pages
	for (week = 1; week < max_week; week++)
	{
		if ((week % 2) == 1)
		{
			# Odd week: left hand page
			# right-hand margin page
			odd_week(week)
		}
		else
		{
			# Even week: right-hand page
			# left-hand margin page
			even_week(week)
		}
	}

	# last left-hand page to force even number of pages
	# Odd week: right-hand margin page
	odd_week(week)

	# document trailer
	print "%%Trailer"
	print "%%Pages:", max_week+1
}


# #####################################
function decode_options()
{
	if (ARGC > 2)
	{
		# decode A size paper
		if (("a4a5a6a7A4A5A6A7" ~ ARGV[2]) && (length(ARGV[2])==2))
		{
			paper = sprintf("A%d", substr(ARGV[2],2))
		}
		# decode A6P paper
		if ((ARGV[2] == "a6p") || (ARGV[2] == "A6P"))
			paper = "A6P"
		# decode B7 paper
		if ((ARGV[2] == "b7") || (ARGV[2] == "B7"))
			paper = "B7"
		# decode Filofax A5 six hole
		if ((ARGV[2] == "f5") || (ARGV[2] == "F5"))
			paper = "F5"
		# decode Filofax Personal six hole
		if ((ARGV[2] == "f6") || (ARGV[2] == "F6"))
			paper = "F6"
		# decode Filofax PersonalA six hole
		if ((ARGV[2] == "f6a") || (ARGV[2] == "F6A"))
			paper = "F6A"
		# decode Filofax Pocket four hole
		if ((ARGV[2] == "p4") || (ARGV[2] == "P4"))
			paper = "P4"
		# decode Filofax Pocket six hole
		if ((ARGV[2] == "p6") || (ARGV[2] == "P6"))
			paper = "P6"
		# decode QL binder paper 3-hole
		if ((ARGV[2] == "q7") || (ARGV[2] == "Q7"))
			paper = "Q7"
		# decode Filofax MiniA 5-hole
		if ((ARGV[2] == "m7") || (ARGV[2] == "M7"))
			paper = "M7"
		# decode Filofax Mini 5-hole
		if ((ARGV[2] == "mi") || (ARGV[2] == "MI"))
			paper = "MI"

		# check for booklet cover
		if ("a4a5a6a6pb7f6p4pm7mi6q7" ~ ARGV[2])
		{
			book = 1
		}
		# check for spine option
		if (ARGC > 3)
			if (ARGV[3] == "spine")
				# enable spine
				spine	= 1

		# check to see if subdivider options set
		if (ARGC > 4)
		{
			hor_sub		= ARGV[4]
			vert_sub	= ARGV[5]
		}
		else
		{
			hor_sub		= 0
			vert_sub	= 0
		}

		# leave only script name and filename in arguments array
		ARGC	= 2
	}
}

function set_page_properties()
{
	# default values
	page_x		= 210				# printed width
	page_y		= 297				# printed height
	max_lines	= 23
	ring_radius	= 2					# binding

	switch (paper)
	{
		case "A4":
			paper_x		= 210				# actual width
			paper_y		= 297				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 80				# centre spacing
			ring_space	= 80				# ring spacing
			ring_group	= 0					# #rings in each group
			ring_radius	= 3					# binding
			break							# end case
		case "A5":
			paper_x		= 148.5				# actual width
			paper_y		= 210				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 69.85				# centre spacing 2.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			ring_radius	= 2.5				# binding
			break							# end case
		case "A6":							# A6 6-hole 1.75" gap
			paper_x		= 105				# actual width
			paper_y		= 148.5				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 38.10				# centre spacing 1.5"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "A6P":							# A6 Filofax Personal punching 6-hole (2" gap)
			paper_x		= 105				# actual width
			paper_y		= 148.5				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "A7":							# A7 stitching
			paper_x		= 74.25				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 3.54				# ring offset 
			ring_gap	= 0					# centre spacing 0 
			ring_space	= 3.89				# ring spacing 
			ring_group	= 8					# #rings in each group
			ring_radius	= 0.177				# sewn binding
			break							# end case
		case "B7":
			paper_x		= 88.29				# actual width (A7 * 4rt(2))
			paper_y		= 124.87			# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 19.05				# centre spacing 0.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "F5":							# Filofax Clipbook A5
			paper_x		= 148.5				# actual width
			paper_y		= 210				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 69.85				# centre spacing 2.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "F6":							# Filofax Personal
			page_x		= 210				# printed width		(Filofax Personal width 3.75", 95mm)
			page_y		= 378				# printed height	(Filofax Personal height 6.75", 171mm)
			paper_x		= 95				# actual width
			paper_y		= 171				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			max_lines	= 27
			break							# end case
		case "F6A":							# Filofax Personal
			page_x		= 210				# printed width		(A6 width 105mm)
			page_y		= 342				# printed height	(Filofax Personal height 6.75", 171mm)
			paper_x		= 105				# actual width
			paper_y		= 171				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			max_lines	= 27
			break							# end case
		case "P4":							# Filofax PocketA 4-hole
			paper_x		= 85.31				# actual width
			paper_y		= 120.65			# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 50.80				# centre spacing 2"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 2					# #rings in each group
			break							# end case
		case "P6":							# Filofax PocketA 6-hole
			paper_x		= 85.31				# actual width
			paper_y		= 120.65			# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 19.05				# centre spacing 0.75"
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "Q7":							# QL wide A7 3-hole
			page_x		= 226				# printed width
			page_y		= 297				# printed height
			paper_x		= 79.90				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 0					# centre spacing 0
			ring_space	= 38.10				# ring spacing 1.5"
			ring_group	= 2					# #rings in each group
			break							# end case
		case "M7":							# Filofax MiniA A7 5-hole
			paper_x		= 74.25				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 0					# centre spacing 0
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		case "MI":							# Filofax Mini narrow A7 5-hole
			page_x		= 190				# printed width
			page_y		= 297				# printed height
			paper_x		= 67.50				# actual width
			paper_y		= 105				# actual height
			ring_margin	= 4.5				# ring offset 
			ring_gap	= 0					# centre spacing 0
			ring_space	= 19.05				# ring spacing 0.75"
			ring_group	= 3					# #rings in each group
			break							# end case
		default:
			paper_x		= 210				# actual width
			paper_y		= 297				# actual height
			ring_margin	= 6					# ring offset 
			ring_gap	= 80				# centre spacing
			ring_space	= 80				# ring spacing
			ring_group	= 0					# #rings in each group
			ring_radius	= 3					# binding
			break							# end case
    }

	# calculate paper scale
	paper_scale	= paper_y/page_y	# printed to actual scale
}

function get_dates()
{
	dates = "JanFebMarAprMayJunJulAugSepOctNovDec"
	# get today's date in known format
	"date +'%a %b %d %T %m %Y'" | getline
	month		= $5
	day			= $3
	thisyear	= $6
	# date string for PostScript creation date (yyyy/mm/dd)
	today		= sprintf("%04d/%02d/%02d", thisyear, month, day)
	# date string for printing record (hh:mm:ss DOW DOM Mon yyyy)
	time		= sprintf("%s %s %s %s %s", $4, $1, $3, $2, $6)

	# get text filedate (can't use FILENAME variable, as doesn't exist in BEGIN section)
	command		= "ls --fu " ARGV[1]
	command | getline
	# check for current year
	year		= substr($6, 1, 4)
	month		= substr($6, 6, 2)
	day			= substr($6, 9, 2)
	filedate	= sprintf("%04d/%02d/%02d", year, month, day)
	filename	= $9

#	# get script file date
#	command 	= "ls --fu " script_path
#	command | getline
#	year		= substr($6, 1, 4)
#	month		= substr($6, 6, 2)
#	day			= substr($6, 9, 2)
#	scriptdate	= sprintf("%04d/%02d/%02d", year, month, day)
}


function print_ps_header()
{
	print "%!PS-Adobe-3.0"
	print "%%Title:		"paper, "Filofax Diary Pages"
	print "%%Creator:		wo2p2.awk", script_version, "copyright 2024-" thisyear, "paranoiaproducts"
	print "%%CreationDate:	" today
	printf("%%%%BoundingBox:	0 0 %d %d\n", page_x * 72/25.4, page_y * 72/25.4)
	print "%%Pages: (atend)"
	print "%%LanguageLevel:	3"
	print "%%EndComments"
	print "%%BeginProlog"
	print "% imposition parameters"
	print "/paper (" paper ") def"
	print "/a4_x\t\t\t" 210, "def"
	print "/a4_y\t\t\t" 297, "def"
	print "/page_x\t\t\t" page_x, "def"
	print "/page_y\t\t\t" page_y, "def"
	print "/paper_x\t\t" paper_x, "def"
	print "/paper_y\t\t" paper_y, "def"
	print "/paper_scale\t" paper_scale, "def"
	print "/ring_margin\t" ring_margin, "def"
	print "/ring_gap\t\t" ring_gap, "def"
	print "/ring_space\t\t" ring_space, "def"
	print "/ring_group\t\t" ring_group, "def"
	print "/ring_radius\t" ring_radius, "def"
	print ""

	# include fixed functions
	system("cat ../scripts/postscript/diary_hdr.eps")
	system("cat ../scripts/postscript/logo.eps")

	print "% end of inserts"
	print "%"
	print "% specific values"
	print "/box_height\t\t" box_height, "def"
	print "/box_width\t\t" box_width, "def"
	print "/hdr_height\t\t" hdr_height, "def"
	print "% spine text (creator copyright, essentially)"
	print "/spine_text (paranoiaproducts)\tdef"

	print ""

	print "%%EndProlog"
	print ""

	# select PostScript document tumble printing mode
	print "%%BeginSetup"
	print "<< /Duplex true /Tumble false >> setpagedevice"
	print ""

	print "%%EndSetup"
}

function title_page()
{
	print ""
	print "%%Page: ", 1, 1
	print "%%BeginPageSetup"
	print "% convert to mm scaling"
	print "72 25.4 div dup scale"
	print "%%EndPageSetup"
	print "% document page", 1
	print "weight setlinewidth"
	print "black"
	# binding holes
	print "paper binding"
	if (spine == 1)
		print "lspine"

	print "% translate to table position"
	print lmargin, footer, "translate"

	# logo
	print t_width / 2, 150, "moveto"
	print 150, "logo"

	# title
	print "/Helvetica-Bold 36 selectfont"
	print t_width / 2, page_y * 0.7, "moveto"
	print "(Diary) centreshow"
	print t_width / 2, page_y * 0.7 - 50, "moveto"
	print "(" the_year ") centreshow"

	print -lmargin, -footer, "translate"

	print "showpage"
}

function month_calendar(show, fill)
{
	# fill highlighted week
	print "% fill week box"
	print "header_grey"
	print 0, cal_d_hgt * fill, "moveto"
	print 0, cal_d_hgt, "rlineto"
	print cal_width, 0, "rlineto"
	print 0, -cal_d_hgt, "rlineto"
	print "closepath"
	print "fill"

	# calendar outline
	print "% outline"
	print "text"
	print 0, 0, "moveto"
	print 0, cal_height, "rlineto"
	print cal_width, 0, "rlineto"
	print 0, -cal_height, "rlineto"
	print "closepath"
	print 0, cal_height - cal_d_hgt, "rmoveto"
	print cal_width, 0, "rlineto"
	print cal_w_wid - cal_width, cal_d_hgt - cal_height, "rmoveto"
	print 0, cal_height, "rlineto"
	print "stroke"

	print "% month calendar"
	print "select_c0_font"
	print cal_w_wid / 2, cal_d_hgt * 5 + cal_t_off, "moveto"
	print "(" substr(cal_days, 1, 2) ")", "centreshow"
	for (day = 1; day < 8; day++)
	{
		print cal_w_wid + cal_d_wid * (day - 0.5), cal_d_hgt * 5 + cal_t_off, "moveto"
		print "(" substr(cal_days, day*2+1, 2) ")", "centreshow"
	}

	# print five week calendar
	for (wk=4; wk>=0 ; wk--)
	{
		# highlight if necessary
		if (wk == fill)
			print "select_cb_font"
		else
			print "select_c0_font"

		# print week number
		the_week	= show - wk
		if (the_week < 1)
			the_week	= the_week+52
		if (the_week > 52)
			the_week	= the_week-52
		print cal_w_wid / 2, cal_d_hgt * wk + cal_t_off, "moveto"
		print "(" the_week ")", "centreshow"

		# print days
		for (day = 1; day < 8; day++)
		{
			print cal_w_wid + cal_d_wid * (day - 0.5), cal_d_hgt * wk + cal_t_off, "moveto"
			print "(" data[show-wk, day] ")", "centreshow"
		}
	}
}

function week_table(week)
{
	# print week table
	print "% table of days"
	print "%	vertical sub-dividers"
	print "text"
	if (vert_sub > 0)
	{
		print "ver_sub_dash				% set dash pattern"
		for (box = 0; box < 2; box++)
			for (line = 1; line < vert_sub; line++)
			{
				print box_width * (box + line / vert_sub), 0, "moveto"
				print 0, t_height, "rlineto"
				print "stroke"
			}
	}
	print "%	horizontal sub-dividers"
	if (hor_sub > 0)
	{
		print "hor_sub_dash				% set dash pattern"
		for (box = 0; box < rows; box++)
		{
			# day boxes
			print 0, box * box_height, "moveto"
			for (line = 1; line < hor_sub; line++)
			{
				print t_width, (box_height - hdr_height) / hor_sub, "rmoveto"
				print -t_width, 0, "rlineto"
			}
			print "stroke"
		}
	}

	# headers
	print "% headers for week & mon-wed"
	print "solid"
	print "header_grey"
	for (line = 1; line <= rows; line++)
	{
		print 0, line * box_height, "moveto"
		print 0, -hdr_height, "rlineto"
		print t_width, 0, "rlineto"
		print 0, hdr_height, "rlineto"
		print "fill"
	}
	print "% horizontal dividers for week & mon-wed"
	print "text"
	for (line = 1; line <= rows; line++)
	{
		print 0, line * box_height, "moveto"
		print t_width, 0, "rlineto stroke"
	}

	# border
	print "% border"
	print "newpath"
	print 0, 0, "moveto"
	print t_width, 0, "rlineto"
	print 0, t_height, "rlineto"
	print -t_width, 0, "rlineto"
	print "closepath"
	print "stroke"

	print "% central divider"
	print "solid"
	print t_width/2, 0, "moveto"
	print 0, t_height, "rlineto stroke"

	# labels
	print "% week"
	print "select_w0_font"		# bold font for week
	# set day position
	print t_mgn_factor * box_width, t_height - 0.7 * hdr_height, "moveto"
	# get weekday text
	print weekdays[0], "show"
	# set date position
	print (1-t_mgn_factor) * box_width, t_height - 0.7 * hdr_height, "moveto"
	if (week < 53)
		print "(" week ") rightshow"				# weeks 1-52
	else
		print "(" week "/" week-52 ") rightshow"	# weeks > 53: show week in next year

	print "% days of week"
	print "select_d0_font"		# normal font for day
	for (row = 0; row < rows; row++)
	{
		if (row > 0)
		{
			# set left day position
			print (t_mgn_factor) * box_width, t_height - row * box_height - 0.7 * hdr_height, "moveto"
			# get weekday text
			print weekdays[2*row], "show"
			# set date position
			print (1-t_mgn_factor) * box_width, t_height - row * box_height - 0.7 * hdr_height, "moveto"
			print "(" data[week, 2*row] ") rightshow"			# date
		}

		# set right day position
		print (1+t_mgn_factor) * box_width, t_height - row * box_height - 0.7 * hdr_height, "moveto"
		# get weekday text
		print weekdays[2*row+1], "show"
		# set date position
		print (2-t_mgn_factor) * box_width, t_height - row * box_height - 0.7 * hdr_height, "moveto"
		print "(" data[week, 2*row+1] ") rightshow"			# date
	}


	# insert events header
	print "% events", month, week, min_date, max_date
}

function odd_week(week)
{
	# Odd week: right-hand margin page
	# page header
	print ""
	print "%%Page: ", week+1, week+1
	print "%%BeginPageSetup"
	print "% convert to mm scaling"
	print "72 25.4 div dup scale"
	print "%%EndPageSetup"
	print "% document page", week+1
	print "% Week", week

	print "weight setlinewidth"
	print "black"
	if (spine == 1)
		print "rspine"

	# table contents
	print "% translate to table"
	print rmargin, footer, "translate"

	# table week fill background
	print "% week fill"
	print "week_grey"
	print 0, t_height, "moveto"
	print box_width, 0, "rlineto"
	print 0, -box_height, "rlineto"
	print -box_width, 0, "rlineto"
	print "closepath"
	print "fill"

	# logo
	print t_width / 2, 150, "moveto"
	print 150, "logo"

	# month calendar in header
	print "% translate to calendar"
	print rcal_x, cal_y, "translate"
	month_calendar(week, 0)					# month behind
	print "% translate back to table"
	print -rcal_x, -cal_y, "translate"

	# print month & date range in header
	# get values
	month		= data[week, "wk_mnth"]
	year		= data[week, "wk_year"]
	min_date	= data[week, 1]
	max_date	= data[week, 7]
	# check for new year transition
	if ((week > 52) && (month == "December") && (min_date > max_date))
	{
		print "% new year transition"
		month	= sprintf("Dec %4d/Jan", year)
		year++
	}

	# week header
	print "text"
	print "% week header"
	print "select_h0_font"
	print box_width * t_mgn_factor, t_height + mnthyr_offset, "moveto"
	print "(" month " " year ") show"

	print "% header range"
	print "select_hd_font"
	print box_width * t_mgn_factor, t_height + range_offset, "moveto"
	printf("(%d-%d) show\n",  min_date,  max_date)	# mon-wed

	week_table(week)

	print "% translate back to origin"
	print -rmargin, -footer, "translate"

	print "showpage"
}

function even_week(week)
{
	# Week: left-hand margin page
	# page header
	print ""
	print "%%Page: ", week+1, week+1
	print "%%BeginPageSetup"
	print "% convert to mm scaling"
	print "72 25.4 div dup scale"
	print "%%EndPageSetup"
	print "% document page", week+1
	print "% Week", week

	print "weight setlinewidth"
	print "black"
	if (spine == 1)
		print "lspine"
	# binding holes
	if ((week % 4) == 0)
		print "paper binding"

	# table contents
	print "% translate to table"
	print lmargin, footer, "translate"

	# table week fill background
	print "% week fill"
	print "week_grey"
	print 0, t_height, "moveto"
	print box_width, 0, "rlineto"
	print 0, -box_height, "rlineto"
	print -box_width, 0, "rlineto"
	print "closepath"
	print "fill"

	# logo
	print t_width / 2, 150, "moveto"
	print 150, "logo"

	# month calendar in header
	print "% translate to calendar"
	print 0, cal_y, "translate"
	month_calendar(week+4, 4)				# month ahead
	print "% translate back to table"
	print 0, -cal_y, "translate"

	# print month & date range in header
	# get values
	month		= data[week, "wk_mnth"]
	year		= data[week, "wk_year"]
	min_date	= data[week, 1]
	max_date	= data[week, 7]
	# check for new year transition
	if ((month == "December") && (min_date < max_date) && (week > 52))
	{
		print "% new year transition"
		month = "January"
		year++
	}
	# check for incorrect transitional month
	if ((month ~ "/") && (min_date < max_date))
		# get month from following week
		month	= data[week+1, "wk_mnth"]

	# week header
	print "text"
	print "% week header"
	print "select_h0_font"
	print t_width - box_width * t_mgn_factor, t_height + mnthyr_offset, "moveto"
	print "(" month " " year ") rightshow"

	print "% header range"
	print "select_hd_font"
	print t_width - box_width * t_mgn_factor, t_height + range_offset, "moveto"
	printf("(%d-%d) rightshow\n",  min_date,  max_date)	# thu-fri

	week_table(week)

	print "% translate back to origin"
	print -lmargin, -footer, "translate"

	print "showpage"
}
